/////////////////////////////////////////////////////////////////////////////////////
// This file is part of the PngOptimizer application
// Copyright (C) 2002/2008 Hadrien Nilsson - psydk.org
// For conditions of distribution and use, see copyright notice in PngOptimizer.h
/////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MainWnd.h"

#include "resource.h"
#include "PngOptimizer.h"
#include "DlgScreenshotsOptions.h"

///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////

bool MainWnd::Create(const wchar* pszTitle, RECT rcWnd)
{
	HINSTANCE hInstance = POApplication::GetInstance().m_hInstance;

	/////////////////////////////////////////////////////////////
	// Register window class
	
	static const wchar szClassName[] = L"PngOptimizer Main Window Class";

	WNDCLASSEXW wcex;
	wcex.cbSize			= sizeof(WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProcStatic;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_MAIN);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szClassName;
	wcex.hIconSm		= NULL;

	ATOM atom = RegisterClassExW(&wcex);
	if( atom == 0)
	{
		m_strErr = L"RegisterClassExW failed";
		return false;
	}

	/////////////////////////////////////////////////////////////
	// Create window

	DWORD nExStyle = WS_EX_TOPMOST;
	DWORD nStyle = WS_OVERLAPPEDWINDOW;

	if( !CreateEx(nExStyle, szClassName, pszTitle, nStyle, rcWnd, NULL))
	{
		m_strErr = L"CreateEx failed";
		return false;
	}
	
	RECT rect = GetClientRect();
	const int nClientWidth = rect.right;
	const int nClientHeight = rect.bottom;

	bool bOk = m_tracectl.Create(0, 0, nClientWidth, nClientHeight, m_hWnd, 1200);
	if( !bOk)
	{
		m_strErr = L"Cannot create trace control";
		return false;
	}

	AllowFileDropping(true);
	
	return true;
}

HWND MainWnd::GetHandle()
{
	return m_hWnd;
}

String MainWnd::GetLastError() const
{
	return m_strErr;
}

void MainWnd::AllowFileDropping(bool bAllow)
{
	if( !m_hWnd)
		return;

	BOOL b2 = bAllow ? TRUE : FALSE;
	DragAcceptFiles(m_hWnd, b2);
}

void MainWnd::AskForScreenshotsDirectory()
{
	DlgScreenshotsOptions dlg;

	POApplication& app = POApplication::GetInstance();

	dlg.m_bUseDefaultDir = app.m_bmpcd.m_bUseDefaultDir;
	dlg.m_strCustomDir = app.m_bmpcd.m_strCustomDir;
	dlg.m_bMaximizeCompression = app.m_bmpcd.m_bMaximizeCompression;
	dlg.m_bAskForFileName = app.m_bmpcd.m_bAskForFileName;

	int nRet = dlg.DoModal(m_hWnd);
	if( nRet == IDCANCEL)
		return;

	app.m_bmpcd.m_bUseDefaultDir = dlg.m_bUseDefaultDir;
	app.m_bmpcd.m_strCustomDir = dlg.m_strCustomDir;
	app.m_bmpcd.m_bMaximizeCompression = dlg.m_bMaximizeCompression;
	app.m_bmpcd.m_bAskForFileName = dlg.m_bAskForFileName;
}

void MainWnd::OnWmMouseWheel(WPARAM wParam, LPARAM lParam)
{
	::SendMessage(m_tracectl.m_hWnd, WM_MOUSEWHEEL, wParam, lParam);
}

void MainWnd::ResizeTraceCtl(int cx, int cy)
{
	m_tracectl.Resize(cx, cy);
}

void MainWnd::SetTraceCtlRedraw(bool bRedraw)
{
	const int nWParam = bRedraw ? TRUE : FALSE;
	::SendMessage(m_tracectl.m_hWnd, WM_SETREDRAW, nWParam, 0);
}

void MainWnd::ClearLine()
{
	m_tracectl.ClearLine();
}

void MainWnd::Clear()
{
	m_tracectl.Clear();

	m_eventListCleared.Fire();
}

void MainWnd::AddLink(const String& strFileNameFinal, const String& strTotalFinalPath)
{
	m_tracectl.AddLink(strFileNameFinal, strTotalFinalPath);
}

void MainWnd::Write(const String& strText, COLORREF cr)
{
	m_tracectl.AddText(strText, cr);
}

void MainWnd::WriteLine(const String& strText, COLORREF cr)
{
	m_tracectl.AddLine(strText, cr);
}

String MainWnd::GetDraggedFileName(HDROP hDrop, int nFile)
{
	String strFileName;

	UINT nLengthNeeded = DragQueryFileW(hDrop, nFile, NULL, 0);
	UINT nRet = DragQueryFileW(hDrop, nFile, strFileName.GetUnsafeBuffer(nLengthNeeded), nLengthNeeded + 1);

	return strFileName;
}


void MainWnd::OnWmDropFiles(WPARAM wParam, LPARAM lParam)
{
	HDROP hDrop = (HDROP) wParam;
	int nFileCount = DragQueryFileW(hDrop, MAX_UINT, NULL, 0);

	m_astrDroppedFilePaths.SetSize(0);
    for(int i = 0; i < nFileCount; ++i)
	{
		m_astrDroppedFilePaths.Add( GetDraggedFileName(hDrop, i));
	}

	DragFinish(hDrop);

	m_eventFilesDropped.Fire(m_astrDroppedFilePaths);
}

bool MainWnd::IsTraceCtlHwnd(HWND hwndCompare)
{
	return hwndCompare == m_tracectl.m_hWnd;
}

/////////////////////////////////////////////////////////////
LRESULT MainWnd::WndProc(UINT nMsg, WPARAM wParam, LPARAM lParam)
{
	POApplication& app = POApplication::GetInstance();

	PAINTSTRUCT ps;
	HDC hdc;
	int cx, cy;

	switch(nMsg) 
	{
	case WM_PAINT:
		hdc = BeginPaint(m_hWnd, &ps);
		EndPaint(m_hWnd, &ps);
		break;
	
	case WM_ERASEBKGND:
		return 1;

	case WM_SIZE:
		cx = LOWORD(lParam);
		cy = HIWORD(lParam);

		ResizeTraceCtl(cx, cy);
		break;

	case WM_DROPFILES:
		OnWmDropFiles(wParam, lParam);
		break;

	case WM_NOTIFY:
		{
			NMHDR* pNM = (NMHDR*) lParam;
			if( IsTraceCtlHwnd(pNM->hwndFrom))
			{
				POINT point;
				GetCursorPos(&point);
				
				HINSTANCE hInstance = POApplication::GetInstance().m_hInstance;
				HMENU hMenu = LoadMenu(hInstance, MAKEINTRESOURCE(IDR_MENU));
				HMENU hSubMenu = GetSubMenu(hMenu, 0);

				HandleMenuItemStates(hSubMenu);

				if( app.m_engine.m_bBackupOldPngFiles)
				{
					CheckMenuItem(hSubMenu, IDM_BACKUP, MF_CHECKED | MF_BYCOMMAND);
				}
				else
				{
					CheckMenuItem(hSubMenu, IDM_BACKUP, MF_UNCHECKED | MF_BYCOMMAND);
				}

				if( app.m_engine.m_bAvoidGreyWithSimpleTransparency)
				{
					CheckMenuItem(hSubMenu, IDM_AVOIDGREYTRANS, MF_CHECKED | MF_BYCOMMAND);
				}
				else
				{
					CheckMenuItem(hSubMenu, IDM_AVOIDGREYTRANS, MF_UNCHECKED | MF_BYCOMMAND);
				}

				if( app.m_engine.m_bKeepInterlacing)
				{
					CheckMenuItem(hSubMenu, IDM_KEEPINTERLACING, MF_CHECKED | MF_BYCOMMAND);
				}
				else
				{
					CheckMenuItem(hSubMenu, IDM_KEEPINTERLACING, MF_UNCHECKED | MF_BYCOMMAND);
				}

				if( app.m_engine.m_bKeepBackgroundColor)
				{
					CheckMenuItem(hSubMenu, IDM_KEEPBACKGROUNDCOLOR, MF_CHECKED | MF_BYCOMMAND);
				}
				else
				{
					CheckMenuItem(hSubMenu, IDM_KEEPBACKGROUNDCOLOR, MF_UNCHECKED | MF_BYCOMMAND);
				}

				TrackPopupMenu(hSubMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, 0, m_hWnd, NULL);
			}
		}
		break;

	case WM_COMMAND:
		{
			int nId = LOWORD(wParam);
			int nEvent = HIWORD(wParam);
			
			if( nId == IDM_SCREENSHOTSOPTIONS)
			{
				AskForScreenshotsDirectory();
				return 0;
			}
			else if( nId == IDM_SHOWSCREENSHOTSDIRECTORY)
			{
				String strDir = app.m_bmpcd.GetDir();
				
				// The / is a valid path, but Windows does not like it, so we convert it
				// into something it likes, "D:\" "E:\" etc.
				strDir = File::GetAbsolutePath(strDir);
				::ShellExecuteW(m_hWnd, L"open", strDir.GetBuffer(), L"", L"", SW_SHOWNORMAL);
				return 0;
			}
			else if( nId == IDM_CLEARLIST)
			{
				// We are allowed to clear the tracectl while the thread is working
				Clear();
				return 0;
			}
			else if( !app.IsJobRunning())
			{
				switch(nId)
				{
				case IDM_BACKUP:
					app.m_engine.m_bBackupOldPngFiles = !app.m_engine.m_bBackupOldPngFiles;
					return 0;

				case IDM_AVOIDGREYTRANS:
					app.m_engine.m_bAvoidGreyWithSimpleTransparency = !app.m_engine.m_bAvoidGreyWithSimpleTransparency;
					return 0;

				case IDM_KEEPINTERLACING:
					app.m_engine.m_bKeepInterlacing = !app.m_engine.m_bKeepInterlacing;
					return 0;

				case IDM_KEEPBACKGROUNDCOLOR:
					app.m_engine.m_bKeepBackgroundColor = !app.m_engine.m_bKeepBackgroundColor;
					return 0;

				case IDM_PASTEFROMCLIPBOARD:
					app.DumpScreenshot();
					return 0;

				default:
					break;
				}
			}
		}
		break;

	case WM_KEYDOWN:
	case WM_MOUSEWHEEL:
		OnWmMouseWheel(wParam, lParam);
		return 0;

	case WM_DESTROY:
		{
			m_eventDestroying.Fire();
		}
		break;
		
	case WM_APP_THREADJOBDONE:
		{
			// The working thread notified us that its job is done
			POApplication::GetInstance().OnJobDone();
		}
		break;

	default:
		return DefWindowProc(m_hWnd, nMsg, wParam, lParam);
	}
	return 0;
}

///////////////////////////////////////////////////////////////////
void MainWnd::HandleMenuItemStates(HMENU hSubMenu)
{
	POApplication& app = POApplication::GetInstance();

	int nEnable = MF_GRAYED;
	if( !app.IsJobRunning())
	{
		nEnable = MF_ENABLED;
	}
	EnableMenuItem(hSubMenu, IDM_BACKUP, nEnable);
	EnableMenuItem(hSubMenu, IDM_AVOIDGREYTRANS, nEnable);
	EnableMenuItem(hSubMenu, IDM_KEEPINTERLACING, nEnable);
	EnableMenuItem(hSubMenu, IDM_KEEPBACKGROUNDCOLOR, nEnable);

	nEnable = MF_GRAYED;

	if( !app.IsJobRunning())
	{
		if( app.IsBmpAvailable())
		{
			nEnable = MF_ENABLED;
		}
	}
	EnableMenuItem(hSubMenu, IDM_PASTEFROMCLIPBOARD, nEnable);
}
