/////////////////////////////////////////////////////////////////////////////////////
// This file is part of the PngOptimizer application
// Copyright (C) 2002/2008 Hadrien Nilsson - psydk.org
// For conditions of distribution and use, see copyright notice in PngOptimizer.h
/////////////////////////////////////////////////////////////////////////////////////

#ifndef PO_MAINWND_H
#define PO_MAINWND_H

#include "POTraceCtl.h"

// PngOptimizer Main window class

class MainWnd : public chuwin32::Window
{
public:
	// Events
	Event0 m_eventDestroying;  // Fired when the window is closing
	Event0 m_eventListCleared; // Fired after the list was cleared
	Event1<const chustd::StringArray&> m_eventFilesDropped; // [astrFilePaths]

public:
	bool Create(const wchar* pszTitle, RECT rcWnd);
	
	HWND   GetHandle();
	String GetLastError() const;
	void   AllowFileDropping(bool bAllow);

	///////////////////////////////////////////////////////////////////////////////
	// Proxy functions to write inside the tracectl
	void Write(const String& strText, COLORREF cr);
	void WriteLine(const String& strText, COLORREF cr);
	void SetTraceCtlRedraw(bool bRedraw);
	void ClearLine();
	void AddLink(const String& strFileNameFinal, const String& strTotalFinalPath);
	///////////////////////////////////////////////////////////////////////////////

private:
	POTraceCtl m_tracectl; // The main control of the window

	String m_strErr;
	StringArray m_astrDroppedFilePaths; // Used as a parameter when firing m_eventFilesDropped

private:
	void AskForScreenshotsDirectory();
	void ResizeTraceCtl(int cx, int cy);
	void HandleMenuItemStates(HMENU hSubMenu);

	bool IsTraceCtlHwnd(HWND hwndCompare);
	void Clear();

	String GetDraggedFileName(HDROP hDrop, int nFile);

	///////////////////////////////////////////////////////
	virtual LRESULT WndProc(UINT nMsg, WPARAM wParam, LPARAM lParam);

	void OnWmMouseWheel(WPARAM wParam, LPARAM lParam);
	void OnWmDropFiles(WPARAM wParam, LPARAM lParam);
	///////////////////////////////////////////////////////
};

#endif // ndef PO_MAINWND_H
