// This file is part of the PngOptimizerCL application
// Copyright (C) 2002/2007 Hadrien Nilsson - psydk.org
// For conditions of distribution and use, see copyright notice in PngOptimizer.h
/////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ArgvParser.h"

ArgvParser::ArgvParser(const String& strCmdLine) : m_strCmdLine(strCmdLine)
{
	Parse();
}

ArgvParser::~ArgvParser()
{

}

const wchar* ArgvParser::JumpOverExeName(const wchar* pszCmdLine)
{
	const int nLength = String::SZLength(pszCmdLine);
	for(int i = 0; i < nLength; ++i)
	{
		wchar c = pszCmdLine[i];
		if( i == 0 && c != '"')
		{
			// No double quote found
			break;
		}

		if( i > 0 && c == '"')
		{
			// Closing double quote found
			return pszCmdLine + i + 1;
		}
	}
	return pszCmdLine;
}


int32 ArgvParser::GetFlagIndex(const String& strFlagName) const
{
	String strWanted = strFlagName.ToLowerCase();

	const int32 nArgCount = m_aArgs.GetSize();
	for(int32 i = 0; i < nArgCount; ++i)
	{
		if( m_aArgs[i].m_bFlag)
		{
			String strName = m_aArgs[i].m_strName.ToLowerCase();

			if( strName == strWanted)
				return i;
		}
	}

	return -1;
}

bool ArgvParser::GetFlag(const String& strFlagName) const
{
	if( GetFlagIndex(strFlagName) < 0)
	{
		return false;
	}
	return true;
}

String ArgvParser::GetFlagString(const String& strFlagName) const
{
	const int32 nIndex = GetFlagIndex(strFlagName);
	if( nIndex < 0)
	{
		return String();
	}

	return m_aArgs[nIndex].m_strFlagString;
}

void ArgvParser::Parse()
{
	bool bInsideArg = false;
	bool bInsideDQ = false;

	const int32 nLength = m_strCmdLine.GetLength();
	const wchar* psz = m_strCmdLine.GetBuffer();
	
	JumpOverExeName(psz);

	StringArray astrArgs;
	StringBuilder sb;

	for(int32 i = 0; i < nLength; ++i)
	{
		wchar c = psz[i];
		if( !bInsideArg)
		{
			if( c != ' ' && c != '\t')
			{
				if( c == '"')
				{
					if( bInsideDQ)
					{
						bInsideDQ = false;
					}
					else
					{
						bInsideDQ = true;
					}
				}

				// New arguments
				sb.Empty();
				sb += c;
				bInsideArg = true;
			}
		}
		else
		{
			// We are inside an argument
			if( c != ' ' && c != '\t')
			{
				if( c == '"')
				{
					if( bInsideDQ)
					{
						bInsideDQ = false;
					}
					else
					{
						bInsideDQ = true;
					}
				}

				sb += c;
			}
			else
			{
				// A separator, on insert it in the argument
				// if we are inside double quotes
				if( bInsideDQ)
				{
					sb += c;
				}
				else
				{
					// End of argument
					astrArgs.Add(sb.ToString());
					bInsideArg = false;
				}
			}
		}
	}
	
	// Add the last string
	astrArgs.Add(sb.ToString());

	ConvertToArgs(astrArgs);
}

void ArgvParser::ConvertToArgs(const StringArray& astrArgs)
{
	const int32 nArgCount = astrArgs.GetSize();
	for(int32 iArg = 0; iArg < nArgCount; ++iArg)
	{
		const String& strArg = astrArgs[iArg];
		const int32 nArgLength = strArg.GetLength();

		if( nArgLength > 0)
		{
			int32 nIndex = m_aArgs.Add();
			Arg& arg = m_aArgs[nIndex];

			wchar cFirst = strArg.GetAt(0);
			if( cFirst == '/' || cFirst == '-')
			{
				StringBuilder sbName;
				StringBuilder sbFlagString;

				// We've got an argument
				arg.m_bFlag = true;

				// Cut the argument
				enum { searchFlagName, searchPreFlagString, searchFlagStringNoDQ, searchFlagStringDQ };
				int8 eSearchMode = searchFlagName;

				for(int32 i = 1; i < nArgLength; ++i)
				{
					wchar c = strArg.GetAt(i);

					if( eSearchMode == searchFlagName)
					{
						if( c == ':')
						{
							eSearchMode = searchPreFlagString;
						}
						else if( c == '"')
						{
							eSearchMode = searchFlagStringDQ;
						}
						else
						{
							sbName += c;
						}
					}
					else
					{
						if( eSearchMode == searchFlagStringNoDQ)
						{
							sbFlagString += c;
						}
						else if( eSearchMode == searchFlagStringDQ)
						{
							if( c == '"')
							{
								// We reached the end of this argument
								break;
							}
							else
							{
								sbFlagString += c;
							}
						}
						else if( eSearchMode == searchPreFlagString)
						{
							if( c == '"')
							{
								eSearchMode = searchFlagStringDQ;
							}
							else
							{
								eSearchMode = searchFlagStringNoDQ;
							}
						}
					}
				}
				arg.m_strName = sbName.ToString();
				arg.m_strFlagString = sbFlagString.ToString();
			}
			else
			{
				arg.m_bFlag = false;
				arg.m_strName = strArg;
			}
		}
	}
}
