// This file is part of the PngOptimizerCL application
// Copyright (C) 2002/2008 Hadrien Nilsson - psydk.org
//
// PngOptimizerCL is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// 
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "../PngOptimizer/POEngine.h"
#include "ArgvParser.h"

class POApplicationConsole
{
public:
	void OnEngineProgressing(const String& str, enum POEngine::TextColor eTextColor)
	{
		Console::Write(str);
	}
};

int32 ChuEntryPoint()
{
	POApplicationConsole app;

	String strCmdLine = Process::GetCommandLine();

	ArgvParser ap(strCmdLine);
	if( ap.GetFlag(L"file"))
	{
		// PngOptimizer.exe -file:"myfile.png"
		// Console mode

		POEngine engine;
		engine.m_eventProgressing.Handle(&app, &POApplicationConsole::OnEngineProgressing);

		engine.m_bBackupOldPngFiles = ap.GetFlag(L"BackupOldPngFiles");
		engine.m_bKeepInterlacing = ap.GetFlag(L"KeepInterlacing");
		engine.m_bAvoidGreyWithSimpleTransparency = ap.GetFlag(L"AvoidGreyWithSimpleTransparency");
		engine.m_bKeepBackgroundColor = ap.GetFlag(L"KeepBackgroundColor");

		String strFilePath = ap.GetFlagString(L"file");
		String strDir, strFileName;
		File::SplitPath(strFilePath, strDir, strFileName);

		StringArray astrFileNames = Directory::GetFileNames(strDir, strFileName);
		engine.OptimizeFiles(strDir, astrFileNames);
	
		return 1;
	}
	
	// Display usage
	Console::WriteLine(L"PngOptimizerCL 1.8 (C) 2002/2008 by Hadrien Nilsson - psydk.org");
	Console::WriteLine(L"Converts GIF, BMP and TGA files to optimized PNG files.");
	Console::WriteLine(L"Optimizes and cleans PNG files.");
	Console::WriteLine(L"");
	Console::WriteLine(L"Usage:");
	Console::WriteLine(L"");
	Console::WriteLine(L"  PngOptimizerCL -file:\"yourfile.png\" [-BackupOldPngFiles] [-KeepInterlacing]   [-AvoidGreyWithSimpleTransparency] [-KeepBackgroundColor]");
	Console::WriteLine(L"");
	Console::WriteLine(L"You can use wildcards to convert/optimize several files at once.");
	Console::WriteLine(L"Values enclosed with [] are optional.");
	Console::WriteLine(L"");
	Console::WriteLine(L"Only files which extension is 'png', 'gif', 'bmp' or 'tga' will be handled.");
	Console::WriteLine(L"This example handles all PNG, GIF, BMP and TGA files of the current directory:");
	Console::WriteLine(L"");
	Console::WriteLine(L"  PngOptimizerCL -file:\"*.*\"");
	Console::WriteLine(L"");
	
	Console::Write(L"Press any key to continue");
	Console::WaitForKey();
	
	return 0;
}
