///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "BitBuffer.h"

using namespace chustd;
//////////////////////////////////////////////////////////////////////
BitBuffer::BitBuffer()
{
	m_pBuffer = null;
	m_nSize = 0;
}

void BitBuffer::SetBuffer(uint8* pBuffer, int32 nByteSize)
{
	m_pBuffer = pBuffer;
	m_nSize = nByteSize;
}

void BitBuffer::ClearBuffer()
{
	for(int32 i = 0; i < m_nSize; ++i)
	{
		m_pBuffer[i] = 0;
	}
}

void BitBuffer::ClearBit(int32 nBitPosition)
{
	const int32 nBytePos = nBitPosition / 8;
	int32 nBitPosInByte = nBitPosition % 8;

	uint8 nValue = 0x80;
	nValue >>= (nBitPosInByte);
	m_pBuffer[nBytePos] &= ~nValue;
}

void BitBuffer::SetBit(int32 nBitPosition)
{
	const int32 nBytePos = nBitPosition / 8;
	int32 nBitPosInByte = nBitPosition % 8;

	uint8 nValue = 0x80;
	nValue >>= (nBitPosInByte);
	m_pBuffer[nBytePos] |= nValue;
}

void BitBuffer::ClearBitRange(int32 nBitPosition, int32 nRange)
{
	// TODO: optimize
	for(int32 i = 0; i < nRange; ++i)
	{
		ClearBit(nBitPosition + i);
	}
}

void BitBuffer::SetBitRange(int32 nBitPosition, int32 nRange)
{
	// TODO: optimize
	for(int32 i = 0; i < nRange; ++i)
	{
		SetBit(nBitPosition + i);
	}
}


int32 BitBuffer::FindFirstZero()
{
	uint8* pBuffer = m_pBuffer;
	const int32 nSize = m_nSize;
	
	for(int32 i = 0; i < nSize; ++i)
	{
		uint8 nByte = pBuffer[i];
		if( nByte != 0xff)
		{
			for(int iBit = 0; iBit < 8; ++iBit)
			{
				uint8 nMask = uint8(0x80 >> iBit); // 1000_0000, 0100_0000, 0010_0000...
				if( (nByte & nMask) == 0)
				{
					// Found
					return i * 8 + iBit;
				}
			}
		}
	}
	return -1;
}

int32 BitBuffer::FindZeroRange(int32 nRange)
{
	uint8* pBuffer = m_pBuffer;
	const int32 nSize = m_nSize;

	int32 nSeqStart = 0;
	int32 nSeqLength = 0;

	for(int32 i = 0; i < nSize; ++i)
	{
		uint8 nByte = pBuffer[i];
		if( nByte != 0xff)
		{
			for(int iBit = 0; iBit < 8; ++iBit)
			{
				uint8 nMask = uint8(0x80 >> iBit); // 1000_0000, 0100_0000, 0010_0000...
				if( (nByte & nMask) == 0)
				{
					if( nSeqLength == 0)
					{
						// Start of sequence
						nSeqStart = i * 8 + iBit;
					}
					nSeqLength++;

					if( nSeqLength == nRange)
					{
						// Found ^^
						return nSeqStart;
					}
				}
				else
				{
					// End of sequence
					nSeqLength = 0;
				}
			}
		}
	}
	return -1;
}

