///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_BMP_H
#define CHUSTD_BMP_H

#include "ImageType.h"

namespace chustd {

class BmpDumpSettings
{
public:
	const void*	m_pBuffer;
	const Palette* m_pPalette;
	int32 m_nWidth;
	int32 m_nHeight;
	PixelFormat	m_epf;

	BmpDumpSettings()
	{
		m_pBuffer = null;
		m_pPalette = null;
		m_nWidth = 0;
		m_nHeight = 0;
		m_epf = pfUnknown;
	}
};

class Bmp : public ImageType  
{
public:
	/////////////////////////////////////////////////////////////
	virtual bool LoadFromFile(IFile& file);
	virtual PixelFormat GetPixelFormat() const;
	virtual const Palette* GetPalette() const;

	virtual String GetLastErrorString() const;
	/////////////////////////////////////////////////////////////

	Bmp();
	virtual ~Bmp();

	// Possible errors for m_nLastError
	enum
	{
		errNotABmpFile = firstErrEnumDerived,
		errBadHeaderSize,
		errUnconsistentDepthAndCompression,
		errUnconsistentDepthAndColorCount,
		errUnsupportedCompressionFormat,
		errUnsupportedBitfieldsFormat,
		errDepthNotSupported,
		errUnexpectedEndOfFile,
		errBadCompressedData,
		errTooManyColors
	};

	static bool IsBmp(IFile& file);

	static bool Dump(const String& strFileName, const BmpDumpSettings& ds);
	static bool Dump(IFile& file, const BmpDumpSettings& ds);

protected:
	enum Compression
	{ 
		compNone, 
		compRle8bits, 
		compRle4bits, 
		compBitfields 
	};

	struct BmpHeader
	{
		uint32 nFileSize; // Complete file size in bytes
		uint32 nReserved;
		uint32 nOffsetToData; // Offset from beginning of the file to the beginning of the image data
		uint32 nBitmapHeaderSize; // 40 bytes
		int32  nWidth;
		int32  nHeight;
		uint16 nPlanes;
		uint16 nDepth; // Bits per pixel. Possible values : 1, 4, 8, 16, 24, 32
		uint32 nCompression; // Possible values : 0=None (BI_RGB), 1=RLE 8bits, 2=RLE 4bits, 3=BI_BITFIELDS
		uint32 nBitmapDataSize; // Rounded to the next 4 byte boundary
		int32  nXPelsPerMeter;
		int32  nYPelsPerMeter;
		int32  nColorCount; // Number of colors
		uint32 nImportantColors;

		void SwapBytes();
	};

	virtual void Initialize();

private:
	PixelFormat m_epf;
	Palette m_ColorTable;

private:
	int32 ComputeBmpBytesPerRow(int32 nExpectedBytesPerRow);
	PixelFormat GetPfFromDepth(uint16 nDepth) const;
	bool RleUncompress(const ByteArray& rbSrc, ByteArray& rbDst, int32 nNiceBytePerRow, int32 nDepth);

	bool ReadPixelData(IFile& file, const BmpHeader& bh);

	bool ReadPixelsFromFile(IFile& file);
	void RemovePadding();
};

} // namespace chustd

#endif // ndef CHUSTD_BMP_H
