///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "Console.h"
#include "String.h"

//////////////////////////////////////////////////////////////////////
using namespace chustd;
//////////////////////////////////////////////////////////////////////

Console::Console()
{

}

Console::~Console()
{

}

void Console::WriteLine(const chustd::String& str)
{
	Write(str + L"\r\n");
}

void Console::Write(const chustd::String& str)
{
#ifdef WIN32
	// Windows functions
	HANDLE hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);

	DWORD nCharWrittenCount = 0;
	DWORD nLength = str.GetLength();
	BOOL bOk = WriteConsoleW(hStdOut, str.GetBuffer(), nLength, &nCharWrittenCount, NULL);
#elif CHUOS

#endif
}



bool Console::WaitForInput(chustd::String& str)
{
	static wchar szPrompt[] = L">";
	return WaitForInput(szPrompt, str);
}

// Wait for a string input by the user
// strPrompt : symbol to print before waiting for the string
// strInput : result
bool Console::WaitForInput(const chustd::String& strPrompt, chustd::String& strInput)
{
	Write(strPrompt);

	wchar szBuffer[128];

#ifdef WIN32
	HANDLE hStdIn = GetStdHandle(STD_INPUT_HANDLE);

	DWORD nNumberOfCharsToRead = (sizeof(szBuffer) / 2) - 1;
	DWORD nNumberOfCharsRead = 0;

	BOOL bWinOk = ReadConsoleW(hStdIn, szBuffer, nNumberOfCharsToRead, &nNumberOfCharsRead, NULL);

	if( !bWinOk)
		return false;
#elif CHUOS

	int32 nNumberOfCharsRead = 0;
#endif

	// Removes any carriage return or line feed
	int32 i = nNumberOfCharsRead - 1;
	for(; i >= 0; --i)
	{
		wchar c = szBuffer[i];
		if( c != 13 && c != 10)
			break;
	}

	int32 nRealCharCount = i + 1;
	strInput.SetBuffer(szBuffer, nRealCharCount);
	return true;
}

uint32 Console::WaitForKey()
{
	uint32 nChar = 0;

#ifdef WIN32
	HANDLE hStdIn = GetStdHandle(STD_INPUT_HANDLE);

	// Switch to raw mode (no line input, no echo input)
	DWORD nOldMode = 0;
	GetConsoleMode(hStdIn, &nOldMode);
	SetConsoleMode(hStdIn, 0);
	
	while(true)
	{
		DWORD nRead = 0;
		INPUT_RECORD inputrec;
		if( !ReadConsoleInputW(hStdIn, &inputrec, 1, &nRead) || (nRead == 0))
		{
			nChar = 0;
			break;
		}
		
		if( (inputrec.EventType == KEY_EVENT) && inputrec.Event.KeyEvent.bKeyDown )
		{
			if( nChar = inputrec.Event.KeyEvent.uChar.UnicodeChar)
			{
				break;
			}
		}
	}
	
	SetConsoleMode(hStdIn, nOldMode);
#endif
	return nChar;
}
