///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_CPPEXTENSION_H
#define CHUSTD_CPPEXTENSION_H

////////////////////////////////
// Set here the byte order of your platform (x86)
namespace chustd {

	enum ByteOrder
	{
		boLittleEndian, // x86
		boBigEndian     // PowerPC
	};
	
	const ByteOrder k_ePlatformByteOrder = boLittleEndian;

} // namespace chustd;

#include "ChuTypes.h"

///////////////////////////////////////////////////////////////////////////
// For arrays ans lists, this macro replaces
//	int32 nCount = m_aSections.GetSize(); 
//	for(int32 i = 0; i < nCount; ++i ) 
// with : 
//  foreach(m_aSections, i) 
#define foreach(varArray, varStep) for(int32 varStep = 0, nCount = varArray.GetSize(); varStep < nCount ; ++varStep) 
///////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////
// Debug helper functions
// This is a very implementation specific part
// Compiles with MS Visual C++ 6.0
// Compiles with G++ (GCC version 3.2)

#if _MSC_VER >= 1400
#define INTERRUPT_EXECUTION() __debugbreak(); // VC8 generic break, works with x86, x64...
#elif _MSC_VER >= 1100
#define INTERRUPT_EXECUTION() __asm { int 3 }     // Works with x86
#else
#define INTERRUPT_EXECUTION() ;
#endif

//////////////////////////////////
#ifndef ASSERT
	#ifdef NDEBUG
		#define ASSERT(exp) ((void)0);
	#else
		#define ASSERT(exp) if(!(exp)) { static bool bOnce = false; if( !bOnce) { bOnce = true;\
			ChustdAssertMessage(__FILE__, __LINE__, 0);\
			INTERRUPT_EXECUTION() } };
	#endif
#endif

//////////////////////////////////
#ifndef VERIFY
	#define VERIFY(exp) if(!(exp)) { static bool bOnce = false; if( !bOnce) { bOnce = true;\
			ChustdAssertMessage(__FILE__, __LINE__, 0);\
			INTERRUPT_EXECUTION() } };
#endif

//////////////////////////////////
#ifndef ASSERTMSG
	#ifdef NDEBUG
		#define ASSERTMSG(exp, str) ((void)0);
	#else
#define ASSERTMSG(exp, psz) if(!(exp)) { static bool bOnce = false; if( !bOnce) { bOnce = true; \
			ChustdAssertMessage(__FILE__, __LINE__, psz);\
			INTERRUPT_EXECUTION() } };
	#endif
#endif

//////////////////////////////////
#ifndef VERIFYMSG
#define VERIFYMSG(exp, psz) if(!(exp)) { static bool bOnce = false; if( !bOnce) { bOnce = true; \
			ChustdAssertMessage(__FILE__, __LINE__, psz);\
			INTERRUPT_EXECUTION() } };
#endif

void* __cdecl operator new(size_t nSize, const char* pszFile, int nLine);
void  __cdecl operator delete(void* pBuffer, const char* pszFile, int nLine);

#ifdef _DEBUG
#define CHUDEBUG_NEW new(__FILE__, __LINE__)
#define new CHUDEBUG_NEW
#define Alloc64(size) Alloc64(size, __FILE__, __LINE__)

#endif // _DEBUG

// Writes traces in an output window if your debugger allows it
void ChustdDebugMessage(const char* pszMsg);

// Warns the user of an assertion failure
void ChustdAssertMessage(const char* pszFileName, int32 nLine, const char* pszMsg);

// Prints the allocated memory blocks
void ChustdDebugPrintLeak(void* p, int32 nSize, const char* pszFileName, int32 nLine);

// Traces a string to a UTF-16 file called chustd-trace.txt
namespace chustd { class String; };
void ChustdTraceLine(const class chustd::String& str);

//#define double please_use_float64_instead_of_double_alone
//#define float please_use_float32_instead_of_float_alone

#endif // ndef CHUSTD_CPPEXTENSION_H
