///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "DynamicMemoryFile.h"

///////////////////////////////////////////////////////////////////////////////
using namespace chustd;
///////////////////////////////////////////////////////////////////////////////

DynamicMemoryFile::DynamicMemoryFile()
{
	m_eByteOrder = boBigEndian;
}

bool DynamicMemoryFile::SetPosition(int64 nOffset, Whence eWhence)
{
	int64 nNewSize = 0;
	if( eWhence == IFile::posBegin)
	{
		nNewSize = nOffset;
	}
	else if( eWhence == IFile::posEnd)
	{
		nNewSize = m_aContent.GetSize() - nOffset;
	}
	else if( eWhence == IFile::posCurrent)
	{
		nNewSize = m_aContent.GetSize() + nOffset;
	}

	if( nNewSize > MAX_INT32)
		return false;

	return m_aContent.SetSize( (int32) nNewSize);
}

int64 DynamicMemoryFile::GetPosition() const
{
	return m_aContent.GetSize();
}

int64 DynamicMemoryFile::GetSize()
{
	return m_aContent.GetCapacity();
}

int32 DynamicMemoryFile::Read(void* pBuffer, int32 nSize)
{
	uint8* pContent = m_aContent.GetBuffer();
	Memory::Copy(pBuffer, pContent + m_aContent.GetSize(), nSize);
	
	SetPosition(nSize, IFile::posCurrent);
	return nSize;
}

int32 DynamicMemoryFile::Write(const void* pBuffer, int32 nSize)
{
	m_aContent.Add( (uint8*)pBuffer, nSize);
	return nSize;
}

ByteOrder DynamicMemoryFile::GetByteOrder() const
{
	return m_eByteOrder;
}

void DynamicMemoryFile::SetByteOrder(ByteOrder eByteOrder)
{
	m_eByteOrder = eByteOrder;
}

bool DynamicMemoryFile::Open(int32 nInitialSize)
{
	return m_aContent.EnsureCapacity(nInitialSize);
}
	
const ByteArray& DynamicMemoryFile::GetContent()
{
	return m_aContent;
}

bool DynamicMemoryFile::WriteFromFile(IFile& fileSrc, int32 nSize)
{
	uint8* pDst = m_aContent.GetBuffer() + GetPosition();
	int32 nRead = fileSrc.Read(pDst, nSize);
	if( nRead != nSize)
	{
		return false;
	}
	return SetPosition(nSize, IFile::posCurrent);
}
