///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_DYNAMICMEMORYFILE_H
#define CHUSTD_DYNAMICMEMORYFILE_H

#include "IFile.h"
#include "Array.h"

namespace chustd {

// Acts like a file, but in memory
class DynamicMemoryFile : public IFile
{
public:
	///////////////////////////////////////////////////////////////////////
	virtual bool  SetPosition(int64 nOffset, Whence eWhence = posBegin);
	virtual int64 GetPosition() const;
	virtual int64 GetSize();
	virtual int32 Read(void* pBuffer, int32 nSize);
	virtual int32 Write(const void* pBuffer, int32 nSize);
	virtual ByteOrder GetByteOrder() const;
	virtual void SetByteOrder(ByteOrder eByteOrder);
	///////////////////////////////////////////////////////////////////////

	// Opens the memory file. Default flags are :
	// Read mode + Binary mode + Big Endian mode
	bool Open(int32 nInitialSize);
	
	// Copies the whole file content to a buffer
	const ByteArray& GetContent();

	// Reads data from an external file and writes it to the DynamicMemoryFile
	bool WriteFromFile(IFile& fileSrc, int32 nSize);

	DynamicMemoryFile();

private:
	ByteArray m_aContent;
	ByteOrder m_eByteOrder;
};

} // namespace chustd

#endif // ndef CHUSTD_DYNAMICMEMORYFILE_H
