///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_FONT_H
#define CHUSTD_FONT_H

#include "Surface8.h"

namespace chustd
{

class String;
class StringArray;
class Int32Array;

class FontImpl;

class Font
{
public:
	Font();
	virtual ~Font();

	bool Create(const chustd::String& strFileName);

	bool SetSize(int32 nSize);
	int32 GetSize() const;
	
	// Line height in pixels : ascent + descent + extra-leading
	int32 GetLineHeight() const;
	
	// Height in pixels from the base line, counting the accents
	int32 GetAscent() const;
	
	// Height in pixels from the base line. Note : positive value
	int32 GetDescent() const;

	// LineHeight + ExtraLeading in pixels
	void SetExtraLeading(int32 nExtraLeading);
	
	// Several text lines height in pixels
	int32 GetTextHeight(const String& strText, int32 nRectWidth);
	
	// Single line width in pixels
	int32 GetTextWidth(const String& strText) const;
	
	// Multi-line text rect in pixels
	void GetTextRect(const String& strText, int32& nRectWidth, int32& nRectHeight);

	// Draws a char in internal buffer and returns a ref to that internal buffer
	const Surface8& RenderChar(uint32 nCodePoint, int32& nOffsetX, int32& nOffsetY, int32& nAdvance) const;

	// Split a string by endlines and blank chars
	chustd::StringArray SplitString(const String& strOriginalText, int32 nRectWidth) const;
	
private:
	
	FontImpl* m_pImpl;

private:
	void GetBlankPositions(const chustd::String& str, chustd::Int32Array& anPositions) const;
};

} // namespace chustd

#endif // ndef CHUSTD_FONT_H
