///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "FormatType.h"
#include "String.h"
#include "Memory.h"

namespace chustd {

void FormatInt32(int32 nValue, wchar szBuffer[12])
{
	szBuffer[0] = '-';
	szBuffer[11] = 0;

	int32 i = 10;
	wchar* psz = szBuffer;
	int32 nNegative = nValue >> 31;
	nValue = (nValue ^ nNegative) + (nNegative & 1);

	do
	{
		int32 nRemain = nValue % 10;
		nValue /= 10;

		psz[i] = wchar( '0' + nRemain );

		--i;
	}
	while(nValue != 0);


	// Shift
	// Start at 1 to keep the '-' (negative number) or start at 0 (positive number) to blow up the '-'
	int32 nOffset = i + nNegative + 1;
	int32 nCount = 11 - i;

	psz = szBuffer + (nNegative & 1);
	do
	{
		psz[0] = psz[nOffset];
		--nCount;
		++psz;
	}
	while( nCount != 0);
}

void FormatUInt32(uint32 nValue, wchar szBuffer[12])
{
	szBuffer[11] = 0;

	uint32 i = 10;
	wchar* psz = szBuffer;

	do
	{
		uint32 nRemain = nValue % 10;
		nValue /= 10;

		psz[i] = char( '0' + nRemain );

		--i;
	}
	while(nValue != 0);


	// Shift
	int32 nOffset = i + 1;
	int32 nCount = 11 - i;

	psz = szBuffer;
	do
	{
		psz[0] = psz[nOffset];
		--nCount;
		++psz;
	}
	while( nCount != 0);
}

//////////////////////////////////////////////////////////
void FormatInt64(int64 nValue, wchar szBuffer[24])
{
	// MAX = 21 counting the final 0
	szBuffer[0] = '-';
	szBuffer[20] = 0;

	int32 i = 19;
	wchar* psz = szBuffer;
	int64 nNegative = nValue >> 63;
	nValue = (nValue ^ nNegative) + (nNegative & 1);

	do
	{
		int32 nRemain = int32(nValue % 10);
		nValue /= 10;

		psz[i] = wchar( '0' + nRemain );

		--i;
	}
	while(nValue != 0);


	// Shift
	// Start at 1 to keep the '-' (negative number) or start at 0 (positive number) to blow up the '-'
	int32 nOffset = i + int32(nNegative) + 1;
	int32 nCount = 21 - i;

	psz = szBuffer + (nNegative & 1);
	do
	{
		psz[0] = psz[nOffset];
		--nCount;
		++psz;
	}
	while( nCount != 0);
}

void FormatUInt64(uint64 nValue, wchar szBuffer[24])
{
	// MAX = 21 counting the final 0
	szBuffer[20] = 0;

	int32 i = 19;
	wchar* psz = szBuffer;

	do
	{
		int32 nRemain = int32(nValue % 10);
		nValue /= 10;

		psz[i] = wchar( '0' + nRemain );

		--i;
	}
	while(nValue != 0);


	// Shift
	// Start at 1 to keep the '-' (negative number) or start at 0 (positive number) to blow up the '-'
	int32 nOffset = i + 1;
	int32 nCount = 21 - i;

	psz = szBuffer;
	do
	{
		psz[0] = psz[nOffset];
		--nCount;
		++psz;
	}
	while( nCount != 0);
}

static void MoveAndFill(wchar* pszBuffer, int8 nWidth, wchar cFill)
{
	int32 nLength = String::SZLength(pszBuffer);
	int32 nMoveCount = nWidth - nLength;
	if( nMoveCount < 0)
		return;

	Memory::Move16(pszBuffer + nMoveCount, pszBuffer, nLength);
	Memory::Set16(pszBuffer, cFill, nMoveCount);
	pszBuffer[nMoveCount + nLength] = 0;
}

void FormatInt32(int32 nValue, wchar* pszBuffer, int8 nWidth, wchar cFill)
{
	FormatInt32(nValue, pszBuffer);
	MoveAndFill(pszBuffer, nWidth, cFill);
}

void FormatInt64(int64 nValue, wchar* pszBuffer, int8 nWidth, wchar cFill)
{
	FormatInt64(nValue, pszBuffer);
	MoveAndFill(pszBuffer, nWidth, cFill);
}

void FormatUInt32(uint32 nValue, wchar* pszBuffer, int8 nWidth, wchar cFill)
{
	FormatUInt32(nValue, pszBuffer);
	MoveAndFill(pszBuffer, nWidth, cFill);
}

void FormatUInt64(uint64 nValue, wchar* pszBuffer, int8 nWidth, wchar cFill)
{
	FormatUInt64(nValue, pszBuffer);
	MoveAndFill(pszBuffer, nWidth, cFill);
}

void FormatUInt32Hex(uint32 nValue, wchar* pszBuffer, int8 nWidth, wchar cFill)
{
	FormatUInt32Hex(nValue, pszBuffer);
	MoveAndFill(pszBuffer, nWidth, cFill);
}

void FormatUInt32Bin(uint32 nValue, wchar* pszBuffer, int8 nWidth, wchar cFill)
{
	FormatUInt32Bin(nValue, pszBuffer);
	MoveAndFill(pszBuffer, nWidth, cFill);
}

void FormatPtr(void* p, wchar* pszBuffer)
{
	const int nSizeofPtr = sizeof(p);
	const int32 nDigitCount = (nSizeofPtr * 2);

	int64 nPtr = int64(p);
	for(int32 i = nDigitCount - 1; i >= 0; --i)
	{
		uint32 nDigit = uint32(nPtr & 0x0f);
		wchar c = 0;
		if( nDigit <= 9)
		{
			c = wchar(nDigit + '0');
		}
		else
		{
			c = wchar((nDigit - 10) + 'a');
		}
		pszBuffer[i] = c;
		nPtr >>= 4;
	}
	pszBuffer[nDigitCount] = 0;
}

void FormatUIntNHex(uint32 nValue, wchar* pszBuffer, int8 nDigitCount)
{
	for(int32 i = 0; i < nDigitCount; ++i)
	{
		uint32 nDigit = (nValue >> (4 * nDigitCount - 4)) & 0x0000000f;
		if( nDigit <= 9)
		{
			nDigit += '0';
		}
		else
		{
			nDigit = (nDigit - 10) + 'a';
		}
		pszBuffer[i] = uint8(nDigit);
		nValue <<= 4;
	}

	pszBuffer[nDigitCount] = 0;
}

void FormatUInt8Hex(uint8 nValue, wchar* pszBuffer)
{
	FormatUIntNHex(nValue, pszBuffer, 2);
}

void FormatUInt16Hex(uint16 nValue, wchar* pszBuffer)
{
	FormatUIntNHex(nValue, pszBuffer, 4);
}

void FormatUInt32Hex(uint32 nValue, wchar* pszBuffer)
{
	FormatUIntNHex(nValue, pszBuffer, 8);
}


void FormatUIntNBin(uint32 nValue, wchar* pszBuffer, int8 nDigitCount)
{
	for(int32 i = 0; i < nDigitCount; ++i)
	{
		uint32 nDigit = (nValue >> (nDigitCount - 1)) & 0x00000001;
		nDigit += '0';
		pszBuffer[i] = uint8(nDigit);
		nValue <<= 1;
	}

	pszBuffer[nDigitCount] = 0;
}

void FormatUInt8Bin(uint8 nValue, wchar* pszBuffer)
{
	FormatUIntNBin(nValue, pszBuffer, 8);
}

void FormatUInt16Bin(uint16 nValue, wchar* pszBuffer)
{
	FormatUIntNBin(nValue, pszBuffer, 16);
}

void FormatUInt32Bin(uint32 nValue, wchar* pszBuffer)
{
	FormatUIntNBin(nValue, pszBuffer, 32);
}

} // namespace chustd
