///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "String.h"
#include "File.h"

#include "ImageLoader.h"

#include "Png.h"
#include "Bmp.h"
#include "Gif.h"
#include "Jpeg.h"
#include "Tga.h"

///////////////////////////////////////////////////////////////////////////////
using namespace chustd;
///////////////////////////////////////////////////////////////////////////////

ImageLoader::ImageLoader() : m_pImageType(null), m_nType(typeNone)
{

}

ImageLoader::~ImageLoader()
{
	delete m_pImageType;
}

bool ImageLoader::Load(const chustd::String& strFileName)
{
	File file;
	if( !file.Open(strFileName))
		return false;

	return LoadFromFile(file);
}

bool ImageLoader::LoadFromFile(IFile& file)
{
	if( !Instanciate(file))
		return false;

	return m_pImageType->LoadFromFile(file);
}

bool ImageLoader::Instanciate(IFile& file)
{
	if( InstanciateLosslessFormat(file) )
	{
		return true;
	}

	int64 nPos = file.GetPosition();
	
	file.SetPosition(nPos);
	if( Jpeg::IsJpeg(file))
	{
		m_pImageType = new Jpeg;
		m_nType = typeJpeg;
		file.SetPosition(nPos);
		return true;
	}

	file.SetPosition(nPos);
	return false;
}

// Exclude Jpeg format (optmize final executable size)
bool ImageLoader::InstanciateLosslessFormat(IFile& file)
{
	delete m_pImageType;
	m_pImageType = null;
	m_nType = typeNone;

	int64 nPos = file.GetPosition();
	
	if( Png::IsPng(file))
	{
		m_pImageType = new Png;
		m_nType = typePng;
		file.SetPosition(nPos);
		return true;
	}
	
	file.SetPosition(nPos);
	if( Gif::IsGif(file))
	{
		m_pImageType = new Gif;
		m_nType = typeGif;
		file.SetPosition(nPos);
		return true;
	}

	file.SetPosition(nPos);
	if( Bmp::IsBmp(file))
	{
		m_pImageType = new Bmp;
		m_nType = typeBmp;
		file.SetPosition(nPos);
		return true;
	}

	file.SetPosition(nPos);
	if( Tga::IsTga(file))
	{
		m_pImageType = new Tga;
		m_nType = typeTga;
		file.SetPosition(nPos);
		return true;
	}

	file.SetPosition(nPos);
	return false;
}
