///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

// This is the common include file for the chustd library, it can be used
// as a precompiled header for compilers which support this feature.

// This header is private and contains OS dependent includes

#ifndef CHUSTD_MASTER_H
#define CHUSTD_MASTER_H

////////////////////////////////
// Remove this include once fpu functions are implemented
#include <math.h> // cos, sin, tan, pow

//////////////////////////////////////////////////////////////////////
#ifdef CHUOS

// ChuOS
#include <chulayer/chulayer.h>
#include <chufs/chufs.h>

const int INVALID_SOCKET = -1;

#elif WIN32

// Windows

//////////////////////////////////////////////////////////////////////
// Avoid calls to undefined function

#define _INC_STRING // string.h declares mem* functions, we do not want them
#define _OLE32PRIV_
#define _OLE32_
#define _SYS_GUID_OPERATORS_
#define __STRALIGN_H_ // Unaligned wide interface (inline call to wcscpy)

// For Windows .h files defining inline functions that use them
extern "C" {
void* memcpy(void* pDst, const void* pSrc, int nSize);
void* memset(void* pDst, int nValue, int nSize);
}
//////////////////////////////////////////////////////////////////////

#ifndef STRICT
#define STRICT 1
#endif

#ifndef UNICODE
#define UNICODE
#endif

// Sockets
#define int32 int // int32 redefined in VC++5 include files
#include <winsock2.h> // Includes <windows.h>
#undef int32
typedef int socklen_t;    // "int" on Windows/VC++, unsigned int on Linux/G++
typedef SOCKET SOCKET_ID; // HANDLE on Windows/VC++, int on Linux/G++

#ifndef INVALID_FILE_ATTRIBUTES
#define INVALID_FILE_ATTRIBUTES ((DWORD)-1)
#endif

#include <windowsx.h>

#ifndef WM_MOUSEWHEEL
#define WM_MOUSEWHEEL 0x020A
#endif

#ifndef GET_WHEEL_DELTA_WPARAM
#define GET_WHEEL_DELTA_WPARAM(wParam) ((short)HIWORD(wParam)) 
#endif

#include <shlobj.h>

// Auto-link
// Does not seem to work, does anybody know why ?
// You may add this lib in your project settings to force linking

// Regular libs : kernel32.lib user32.lib gdi32

// For drag-and-drop
#pragma comment(lib, "shell32.lib")

// For sockets
#pragma comment(lib, "ws2_32.lib")

#define Rectangle chustd_Rectangle

#undef MOD

// Missing definition for Win32/SetFilePointer
#ifndef INVALID_SET_FILE_POINTER 
#define INVALID_SET_FILE_POINTER ((DWORD)-1)
#endif

#undef GetFirstChild // Otherwise, conflict with XmlDocument

#else

// Other OS... Linux ?

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h> // for close()
const int INVALID_SOCKET = -1;
typedef int SOCKET_ID;

#endif // _WIN32
//////////////////////////////////////////////////////////////////////

#if _MSC_VER
#pragma comment(linker, "/nodefaultlib")
#endif

// Common header not OS dependant
#include "CppExtension.h"

#endif // ndef CHUSTD_MASTER_H
