///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_MEMORY_H
#define CHUSTD_MEMORY_H

namespace chustd {

class Memory
{
public:
	static void Move(void* pDst, const void* pSrc, int32 nCount);
	static void Move16(void* pDst, const void* pSrc, int32 nCount);
	
	static void Copy(void* pDst, const void* pSrc, int32 nByteCount);
	static void Copy16(void* pDst, const void* pSrc, int32 nCount);
	static void Copy32(void* pDst, const void* pSrc, int32 nCount);
	static void Copy64(void* pDst, const void* pSrc, int32 nCount);
	static void Copy64From8(void* pDst, const void* pSrc, int32 nByteCount);
	static void Copy64From16(void* pDst, const void* pSrc, int32 nUInt16Count);
	
	static void Swap(void* pDst, void* pSrc, int32 nCount);

	static void Zero(void* pDst, int32 nByteCount);
	static void Zero16(void* pDst, int32 nCount);
	static void Zero32(void* pDst, int32 nCount);
	
	static void Set(void* pDst, uint8 nValue, int32 nByteCount);
	static void Set16(void* pDst, uint16 nValue, int32 nCount);
	static void Set32(void* pDst, uint32 nValue, int32 nCount);
	
	static bool IsEqual(const void* pSrc0, const void* pSrc1, int32 nByteCount);
	static bool IsEqual32(const void* pSrc0, const void* pSrc1, int32 nCount);

	static inline int32 ByteCountToInt64Count(int32 nSizeInBytes)
	{
		const int32 nRemainBit0 = (nSizeInBytes >> 0) & 1;
		const int32 nRemainBit1 = (nSizeInBytes >> 1) & 1;
		const int32 nRemainBit2 = (nSizeInBytes >> 2) & 1;
		const int32 nChunkCount = (nSizeInBytes >> 3) + (nRemainBit2 | nRemainBit1 | nRemainBit0);

		return nChunkCount;
	}

	static bool Is16BitsAligned(const void* p);
	static bool Is32BitsAligned(const void* p);
	static bool Is64BitsAligned(const void* p);
	
#undef Alloc64

	// Allocation of 64 bits blocks
	static void* Alloc64(int32 nByteCount);
	static void* Alloc64(int32 nByteCount, const char* pszFileName, int32 nLine);
	static void Free64(void* pData);

#define Alloc64(size) Alloc64(size, __FILE__, __LINE__)

	static uint32 GetAllocated();
	static uint32 GetPeak();

	static void DebugPrintBlocks();

	static void InitAllocator();
};

template <int32 t_nCount>
class AlignedStackBuffer64Bits
{
public:
	uint64* GetBuffer()
	{
		uint8* pBuf8 = (uint8*) m_aBuffer;
		int nLowPart = pBuf8 - (uint8*)(0);
		pBuf8 += ((8 - nLowPart) & 0x07);
		return (uint64*)pBuf8;
	}
	int32 Sizeof() const { return t_nCount * 8; }
private:
	uint64 m_aBuffer[ t_nCount + 8];
};

} // namespace chustd

#endif // ndef CHUSTD_MEMORY_H
