///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "Process.h"
#include "String.h"
#include "Array.h"
#include "File.h"

using namespace chustd;

String Process::GetCurrentDirectory()
{
#ifdef WIN32
	// Includes ending 0
	const int nSizeofBuffer = ::GetCurrentDirectory(0, NULL);

	wchar* pBuffer = new wchar[nSizeofBuffer];
	::GetCurrentDirectory(nSizeofBuffer, pBuffer);

	String strDirectory(pBuffer);

	delete[] pBuffer;
	return strDirectory;

#elif CHUOS
	return String();
#endif
}

void Process::SetCurrentDirectory(const String& strDirectory)
{
#ifdef WIN32
	::SetCurrentDirectory(strDirectory.GetBuffer());
#elif CHUOS
	
#endif
}

uint16 Process::GetCurrentId()
{
#ifdef WIN32
	uint32 nId = ::GetCurrentProcessId();
	return uint16(nId);
#elif CHUOS
	return 0;	
#endif
}

String Process::GetCommandLine()
{
#ifdef WIN32
	LPWSTR psz = ::GetCommandLineW();
	return String(psz);
#elif CHUOS
	return String();	
#endif
}

chustd::StringArray Process::CommandLineToArgv(const String& strCmdLine)
{
	StringArray astrResult;
	astrResult.SetAsReturnValue();
	
	enum { stateInsideArg, stateInsideQuotedArg, stateCalm};
	
	int nState = stateCalm;
	int32 nArgStart = -1;

	const int32 nLength = strCmdLine.GetLength();
	for(int32 iChar = 0; iChar < nLength; ++iChar)
	{
		wchar c = strCmdLine.GetAt(iChar);
		if( nState == stateCalm)
		{
			if( c == L'"')
			{
				nState = stateInsideQuotedArg;
				nArgStart = iChar + 1;
			}
			else if( c != L' ')
			{
				nState = stateInsideArg;
				nArgStart = iChar;
			}
		}
		else if( nState == stateInsideQuotedArg)
		{
			if( c == L'"')
			{
				// End
				ASSERT(nArgStart >= 0);
				String strArg = strCmdLine.Mid(nArgStart, iChar - nArgStart);
				astrResult.Add(strArg);
				nState = stateCalm;
			}
		}
		else if( nState == stateInsideArg)
		{
			if( c == L' ')
			{
				ASSERT(nArgStart >= 0);
				String strArg = strCmdLine.Mid(nArgStart, iChar - nArgStart);
				astrResult.Add(strArg);
				nState = stateCalm;
			}
		}
	}

	// Last argument
	if( nState == stateInsideArg)
	{
		ASSERT(nArgStart >= 0);
		String strArg = strCmdLine.Mid(nArgStart);
		astrResult.Add(strArg);
		nState = stateCalm;
	}

	return astrResult;
}

String Process::GetExecutablePath()
{
	StringArray astrArgs = Process::CommandLineToArgv( GetCommandLine());
	if( astrArgs.GetSize() == 0)
	{
		return String();
	}
	return astrArgs[0];
}

String Process::GetExecutableDirectory()
{
	String strPath = GetExecutablePath();
	
	String strDir, strFileName;
	File::SplitPath(strPath, strDir, strFileName);
	return strDir;
}

////////////////////////////////////////////////
void Thread::Sleep(uint32 nDelayMs)
{
#ifdef WIN32
	::Sleep(nDelayMs);
#elif CHUOS
	
#endif
}
