///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_PTRARRAY_H
#define CHUSTD_PTRARRAY_H

#include "Array.h"

namespace chustd {

// Simple array of pointers
template <class T> class PtrArrayNoDelete : public Array<void*>
{
public:
	T*& operator [] (int32 nIndex)
	{
		return (T*&) (m_paTs[nIndex]);
	}

	T* operator [] (int32 nIndex) const
	{
		return (T*) (m_paTs[nIndex]);
	}
};


// Manages a pointers array
// The array owns the data, so it deletes the pointed data
template <class T> class PtrArray : public Array<T*>
{
public:
	int32 Add(T* pT)
	{
		// Every address must be unique
		if(pT)
		{
			ASSERT( Find(pT) < 0);
		}

		return Array<T*>::Add(pT);
	}

	int32 Add(T* const * papTs, int32 nCount) {	return Array<T*>::Add(papTs, nCount); }
	int32 Add(const PtrArray<T>& arr) { return Array<T*>::Add(arr); }

	PtrArray<T>& operator = (const PtrArray<T>& arr);

	PtrArray();
	PtrArray(const PtrArray<T>& arr);
	virtual ~PtrArray();

protected:
	virtual void DeletePointedData(int32 nStartIndex, int32 nCount);
};

template <class T>
PtrArray<T>::PtrArray()
{
}

// Copy constructor
template <class T>
PtrArray<T>::PtrArray(const PtrArray<T>& arr)
{
	if( arr.IsReturnedByValue())
	{
		m_paTs = arr.m_paTs;
		m_nCount = arr.m_nCount;
		m_nRbvAndAllocated = arr.GetCapacity(); // Set the rbv bit to 0

		// Const cast, bad, but well, advandced C++
		CPtrArray<T>& arr2 = (CPtrArray<T>&) arr;

		arr2.m_paTs = null;
		arr2.m_nCount = 0;
		arr2.m_nRbvAndAllocated = 0;
	}
	else
	{
		m_paTs = null;
		m_nCount = 0;
		m_nRbvAndAllocated = 0;

		Add(arr.m_paTs, arr.m_nCount);

		// Create copies of pointed data
		for(int32 i = 0; i < m_nCount; i++)
		{
			m_paTs[i] = new T( *m_paTs[i]);
		}
	}
}


// Assignment operator
template <class T>
PtrArray<T>& PtrArray<T>::operator = (const PtrArray<T>& arr)
{
	Array<T*>::operator = (arr);
	
	// Create copies of pointed data
	for(int32 i = 0; i < m_nCount; i++)
	{
		m_paTs[i] = new T( *m_paTs[i]);
	}

	return *this;
}

template <class T>
PtrArray<T>::~PtrArray()
{
	for(int32 i = 0; i < m_nCount; ++i)
	{
		delete m_paTs[i];
	}
}

template <class T>
void PtrArray<T>::DeletePointedData(int32 nStartIndex, int32 nCount)
{
	for(int32 i = 0; i < nCount; i++)
	{
		T* pElement = m_paTs[nStartIndex + i];
		delete pElement;
	}
}

} // namespace chustd

#endif // ndef CHUSTD_PTRARRAY_H
