///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "ReadOnlySurface8.h"
#include "ImageType.h"

//////////////////////////////////////////////////////////////////////
using namespace chustd;
//////////////////////////////////////////////////////////////////////

bool ReadOnlySurface8::Create(int32 nWidth, int32 nHeight)
{
	return super::Create(nWidth, nHeight, nWidth, 8);
}

void ReadOnlySurface8::Create(const ReadOnlySurface8& surfSource, const Rectangle& rect)
{
	SetBufferHelper(surfSource, rect);
}

bool ReadOnlySurface8::Create(const ImageType& it)
{
	bool bOk = Create(it.GetWidth(), it.GetHeight());
	if( !bOk)
		return false;

	uint8* pSrc = (uint8*) it.GetBuffer();
	PixelFormat pfSrc = it.GetPixelFormat();

	uint8* pDst = (uint8*) GetBuffer();

	const int32 nWidth = GetWidth();
	const int32 nHeight = GetHeight();
	const int32 nPixelCount = nWidth * nHeight;
	
	if( pfSrc == pf8bppGrayScale)
	{
		for(int32 i = 0; i < nPixelCount; i++)
		{
			const uint8 nIndex = pSrc[i];
			pDst[0] = nIndex;
			pDst += 1;
		}
	}
	else if( pfSrc == pf16bppGrayScale)
	{
		for(int32 i = 0; i < nPixelCount; i++)
		{
			uint32 g = uint16( (pSrc[0] << 8) | pSrc[1]);
			pSrc += 2;

			// Magic 16 bits -> 8 bits conversion
			g = (g >> 8) + ((g >> 7) & 1);
			g -= (g >> 8);

			pDst[0] = uint8(g);
			pDst += 1;
		}
	}
	else if( pfSrc == pf1bppGrayScale || pfSrc == pf2bppGrayScale || pfSrc == pf4bppGrayScale)
	{
		int32 nBitsPerPix = ImageType::SizeofPixelInBits(pfSrc);

		//////////////////////////////////////////////////////////////////////////////////////////
		// Create the table to convert the indexes stored with less than 8 bits into 8 bits values
		const int32 nColCount = 1 << nBitsPerPix;
		const int32 nBitBlockCount = 8 / nBitsPerPix;
		uint8 aConvertTable[16];
		
		for(int32 iCol = 0; iCol < nColCount; ++iCol)
		{
			uint8 nGrey = 0;
			for(int32 iBit = 0; iBit < nBitBlockCount; ++iBit)
			{
				nGrey <<= nBitsPerPix;
				nGrey |= iCol;
			}

			aConvertTable[iCol] = nGrey;
		}

		//////////////////////////////////////////////////////////////////////////////////////////

		int32 nMask = ~((~0L) << nBitsPerPix);

		int32 nShift = 8 - nBitsPerPix;
		for(int32 y = 0; y < nHeight; ++y)
		{
			for(int32 x = 0; x < nWidth; ++x)
			{
				const uint8 nByte = pSrc[0];
						
				int32 nIndex = (nByte >> nShift) & nMask;
				const uint8 nGray = aConvertTable[nIndex];

				nShift -= nBitsPerPix;
				if( nShift < 0)
				{
					nShift = 8 - nBitsPerPix;
					pSrc += 1;
				}

				pDst[0] = nGray;
				pDst += 1;
			}
			
			// Some padding bits remain
			if( nShift != 8 - nBitsPerPix)
			{
				nShift = 8 - nBitsPerPix;
				pSrc += 1;
			}
		}
	}
	else
	{
		// Unsupported pixel format
		return false;
	}
	return true;
}
