///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "Rectangle.h"

using namespace chustd;

void Rectangle::ClipIn(const Rectangle& rect)
{
	///////////////////////////////////
	// Ce sera plus lisible de travailler 
	// avec des noms plus courts ^^
	int32& nX = m_ptTopLeft.m_nX;
	int32& nY = m_ptTopLeft.m_nY;
	int32& nWidth = m_dimension.m_nWidth;
	int32& nHeight = m_dimension.m_nHeight;
	const int32& nX2 = GetX2();
	const int32& nY2 = GetY2();

	const int32& nRectX = rect.m_ptTopLeft.m_nX;
	const int32& nRectY = rect.m_ptTopLeft.m_nY;
	const int32& nRectWidth = rect.m_dimension.m_nWidth;
	const int32& nRectHeight = rect.m_dimension.m_nHeight;
	const int32& nRectX2 = rect.GetX2();
	const int32& nRectY2 = rect.GetY2();

	// On va vrifier les 4 cts de notre rectangle
	// et les ramener dans les bornes si besoin

	// Haut
	if( nY < nRectY )
	{
		const int32 nDiffY = nRectY - nY;
		nY += nDiffY;
		nHeight -= nDiffY;
	}

	// Bas
	if( nY2 > nRectY2 )
	{
		const int32 nDiffY = nY2 - nRectY2;
		nHeight -= nDiffY;
	}

	// Gauche
	if( nX < nRectX )
	{
		const int32 nDiffX = nRectX - nX;
		nX += nDiffX;
		nWidth -= nDiffX;
	}

	// Droite
	if( nX2 > nRectX2 )
	{
		const int32 nDiffX = nX2 - nRectX2;
		nWidth -= nDiffX;
	}

	// Rectification en cas d'incohrence dans le rectangle

	if( nX < nRectX )
	{
		nX = nRectX;
	}
	
	if( nY < nRectY )
	{
		nY = nRectY;
	}
	
	if( nWidth < 0 )
	{
		nWidth = 0;
	}

	if( nHeight < 0 )
	{
		nHeight = 0;
	}

	if( nWidth > nRectWidth )
	{
		nWidth = nRectWidth;
	}

	if( nHeight > nRectHeight )
	{
		nHeight = nRectHeight;
	}
}

void Rectangle::SetCoords(int32 nX, int32 nY, int32 nX2, int32 nY2)
{
	m_ptTopLeft.m_nX = nX;
	m_ptTopLeft.m_nY = nY;

	m_dimension.m_nWidth = nX2 - nX + 1;
	m_dimension.m_nHeight = nY2 - nY + 1;
}

Point Rectangle::GetPointTopLeft() const
{
	return m_ptTopLeft;
}

Point Rectangle::GetPointTopRight() const
{
	return Point(m_ptTopLeft.m_nX + m_dimension.m_nWidth - 1, m_ptTopLeft.m_nY);
}

Point Rectangle::GetPointBottomLeft() const
{
	return Point(m_ptTopLeft.m_nX, m_ptTopLeft.m_nY + m_dimension.m_nHeight - 1);
}

Point Rectangle::GetPointBottomRight() const
{
	return Point(m_ptTopLeft.m_nX + m_dimension.m_nWidth - 1, 
				m_ptTopLeft.m_nY + m_dimension.m_nHeight - 1
				);
}

void Rectangle::Shrink(int32 nAmountX, int32 nAmountY)
{
	m_ptTopLeft.m_nX += nAmountX;
	m_ptTopLeft.m_nY += nAmountY;
	m_dimension.m_nWidth -= 2 * nAmountX;
	m_dimension.m_nHeight -= 2 * nAmountY;

	if( m_dimension.m_nWidth < 0 ) 
	{
		m_dimension.m_nWidth = 0;
	}
	if( m_dimension.m_nHeight < 0 ) 
	{
		m_dimension.m_nHeight = 0;
	}
}

void Rectangle::Translate(int32 nAmountX, int32 nAmountY)
{
	m_ptTopLeft.m_nX += nAmountX;
	m_ptTopLeft.m_nY += nAmountY;
}

bool Rectangle::Contains(const Point& pt) const
{
	const Point ptBottomRight = GetPointBottomRight();

	const int32 nX = pt.m_nX;
	const int32 nY = pt.m_nY;
	const int32 nRectX = m_ptTopLeft.m_nX;
	const int32 nRectY = m_ptTopLeft.m_nY;
	const int32 nMaxX = ptBottomRight.m_nX;
	const int32 nMaxY = ptBottomRight.m_nY;

	if( (nRectX <= nX && nX <= nMaxX) && (nRectY <= nY && nY <= nMaxY) )
	{
		return true;
	}
	return false;
}

bool Rectangle::IsEmpty() const
{
	return m_dimension.m_nWidth == 0 && m_dimension.m_nHeight == 0;
}

void Rectangle::Empty()
{
	Set(0, 0, 0, 0);
}

void Rectangle::MergeWith(const Rectangle& rect)
{
	if( rect.IsEmpty())
	{
		// No merge
		return;
	}

	if( IsEmpty())
	{
		*this = rect;
		return;
	}

	const int32 nThisX1 = m_ptTopLeft.m_nX;
	const int32 nThisY1 = m_ptTopLeft.m_nY;
	const int32 nThisX2 = GetX2();
	const int32 nThisY2 = GetY2();

	const int32 nX1b = rect.m_ptTopLeft.m_nX;
	const int32 nY1b = rect.m_ptTopLeft.m_nY;
	const int32 nX2b = rect.GetX2();
	const int32 nY2b = rect.GetY2();

	int32 nX1c, nY1c, nX2c, nY2c;

	if( nX1b < nThisX1)
	{
		nX1c = nX1b;
	}
	else
	{
		nX1c = nThisX1;
	}

	if( nY1b < nThisY1)
	{
		nY1c = nY1b;
	}
	else
	{
		nY1c = nThisY1;
	}

	if( nX2b > nThisX2)
	{
		nX2c = nX2b;
	}
	else
	{
		nX2c = nThisX2;
	}

	if( nY2b > nThisY2)
	{
		nY2c = nY2b;
	}
	else
	{
		nY2c = nThisY2;
	}

	m_ptTopLeft.m_nX = nX1c;
	m_ptTopLeft.m_nY = nY1c;
	m_dimension.m_nWidth = (nX2c - nX1c) + 1;
	m_dimension.m_nHeight = (nY2c - nY1c) + 1;
}
