///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_RECTANGLE_H
#define CHUSTD_RECTANGLE_H

#include "Point.h"
#include "Dimension.h"

namespace chustd {

class Rectangle  
{
public:
	Point m_ptTopLeft;
	Dimension m_dimension;

public:
	int32 GetX2() const { return m_ptTopLeft.m_nX + m_dimension.m_nWidth - 1; }
	int32 GetY2() const { return m_ptTopLeft.m_nY + m_dimension.m_nHeight - 1; }

	Point GetPointTopLeft() const;
	Point GetPointTopRight() const;
	Point GetPointBottomLeft() const;
	Point GetPointBottomRight() const;
	
	void SetCoords(int32 nX, int32 nY, int32 nX2, int32 nY2);

	void ClipIn(const Rectangle& rect);

	void Translate(int32 nAmountX, int32 nAmountY);
	void Shrink(int32 nAmountX, int32 nAmountY);

	bool Contains(const Point& pt) const;
	bool IsEmpty() const;
	void Empty();

	// Enlarge the rectangle to include the rectangle in argument
	void MergeWith(const Rectangle& rect);

public:
	Rectangle(int32 nX, int32 nY, int32 nWidth, int32 nHeight)
	{
		m_ptTopLeft.m_nX = nX;
		m_ptTopLeft.m_nY = nY;
		m_dimension.m_nWidth = nWidth;
		m_dimension.m_nHeight = nHeight;
	}

	Rectangle()
	{
		m_ptTopLeft.m_nX = 0;
		m_ptTopLeft.m_nY = 0;
		m_dimension.m_nWidth = 0;
		m_dimension.m_nHeight = 0;
	}

	Rectangle(const Rectangle& rect)
	{
		m_ptTopLeft.m_nX = rect.m_ptTopLeft.m_nX;
		m_ptTopLeft.m_nY = rect.m_ptTopLeft.m_nY;
		m_dimension.m_nWidth = rect.m_dimension.m_nWidth;
		m_dimension.m_nHeight = rect.m_dimension.m_nHeight;
	}

	void Set(int32 nX, int32 nY, int32 nWidth, int32 nHeight)
	{
		m_ptTopLeft.m_nX = nX;
		m_ptTopLeft.m_nY = nY;
		m_dimension.m_nWidth = nWidth;
		m_dimension.m_nHeight = nHeight;
	}


	Rectangle(const Point& ptTopLeft, const Dimension& dim) : m_ptTopLeft(ptTopLeft), m_dimension(dim)
	{
	}
};

}

#endif // ndef CHUSTD_RECTANGLE_H
