///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_SOCKET_H
#define CHUSTD_SOCKET_H

namespace chustd {

typedef void (*PFN_ASYNCHANDLER)(int32, void*);

class String;

// Base class for specialized socket classes
class Socket
{
public:
	int32 Receive(uint8* pBuffer, int32 nBufferSize, bool bBlockUntilSizeReached);
	int32 Send(const uint8* pBuffer, int32 nBufferSize, bool bBlockUntilSizeReached);

	// The handler is called when data is pending in the socket, which should be
	// retrieved with a call to Receive
	int32 AsyncReceive(uint8* pBuffer, int32 nBufferSize, PFN_ASYNCHANDLER pfnHandler);

	// The handler is called when the socket is ready to send data, which should be
	// sent with a call to Send
	int32 AsyncSend(const uint8* pBuffer, int32 nBufferSize, PFN_ASYNCHANDLER pfnHandler);

	// Returns the socket handle, which can then be used in some other network functions
	// Returns -1 if no socket ID available
	int64 GetSocketID() const;

	// Not static as this function should be available only when at least one CSocket constructor was called
	bool IsNetworkWorking() const { return ms_bNetworkWorking; }

	void Close();

	Socket();
	virtual ~Socket();

	////////////////////////////////////////////////////

	// Converts an text ip address into a 32 bits value.
	// The 32 bits value byte order is plateform dependant.
	// Returns false if the ip adress is not valid
	static bool StringToIpAddress(const chustd::String& strIpAddress, uint32& nIpAddress);

	// Converts a 32 bits ip address into a string
	// The 32 bits value byte order is plateform dependant.
	static String IpAddressToString(uint32 nIpAddress);

protected:
	int64 m_nIDSocket;	// OS Socket Handle (ULONG_PTR on Win32 => 64 bits on Win64)
	uint8 m_nType;      // Type

	static int ms_nSocketStartupCount;
	static bool ms_bNetworkWorking; // Set to false if initialization failed

protected:
	friend class TcpServer;

	bool IsSendBufferFlooded() const;
	bool BlockUntilSendPossible(); // Returns true if ok

	enum Type
	{
		typeUnknown,
		typeTcp,
		typeUdp
	};

	bool Open(Type eType);
	bool IsOpen() const;
};

} // namespace chustd

#endif // ndef CHUSTD_SOCKET_H
