///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_SORT_H
#define CHUSTD_SORT_H

#include "Memory.h"

namespace chustd {

namespace Sort
{
	void ByteSortAtt(uint32* paElements, uint8* paSortAttributes, int32 nSize);
	void ByteSortAtt(uint32* paElements, uint32* paSortAttributes, int32 nSize);
	
	// in/out :  paElements : the elements to sort
	//          paSortAttributs : the numbers associated to each element which
	//          are to be used in the sort
	void ByteSortAtt(uint32* paElements, uint32* paSortAttributes, 
		uint32* paTmpElements, uint32* paTmpSortAttributs, int32 nSize);

	void ByteSort(const uint8* pIn, uint8* pOut, int32 nSize);
	void ByteSort(uint32* pIn, uint32* pOut, int32 nSize);
	
	void ByteSortLittleEndian(const uint32* pIn, uint32* pOut1, uint32* pOut2, int32 nSize);

	void BubbleSort(uint32* pa, int32 nSize);

	inline uint8 Byte( uint32 n, uint8 nShift )
	{
		return uint8( n >> (nShift << 3) );
	}

	template <class T>
	void ByteSort(T* paElementsIn, T* paElementsOut, uint8* paSortAttributes, int32 nSize)
	{
		uint32 aOccurrences[256];
		
		Memory::Zero32(aOccurrences, 256);
			
		for( int32 i = 0; i < nSize; i++ )
		{
			const uint8 nByte = paSortAttributes[i];
			aOccurrences[nByte]++;
		}

		// Compute final positions in the sorted array
		uint32 nPosition = 0;
		for( int32 i = 0; i < 256; i++ )
		{			
			const uint32 nIncrement = aOccurrences[i];
			aOccurrences[i] = nPosition;
			nPosition += nIncrement;
		}

		for( int32 i = 0; i < nSize; i++ )
		{			
			const uint8 nByte = paSortAttributes[i];
			nPosition = aOccurrences[nByte];
			aOccurrences[nByte]++;

			paElementsOut[nPosition] = paElementsIn[i];
		}
	}

}

} // namespace chustd

#endif // ndef CHUSTD_SORT_H
