///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "StringData.h"

using namespace chustd;

/////////////////////////////////////////////////////////////////////////////////////
uint16* StringData::CreateInstance(int32 nWantedLength)
{
	ASSERT(nWantedLength >= 0);

	///////////////////////////////////////////////////////
	// Number of uint64 to allocate
	// 1 is the two int32 (length and ref)
	const int32 nTotal = 1 + GetAllocatedChunkCount(nWantedLength);

	uint64* pAlloc = (uint64*) Memory::Alloc64(nTotal * 8);
	ASSERT(pAlloc); // Raise an exception ?
	
	StringData* pInst = (StringData*) pAlloc;
	pInst->m_nLength = nWantedLength; // Character count (0 not included)
	pInst->m_nRef = 1;
	pInst->m_szBuffer[nWantedLength] = 0; // Puts a final 0 after the last used char
	///////////////////////////////////////////////////////

	return pInst->m_szBuffer;
}

// New instance copy of StringData pointed by pData
uint16* StringData::CreateInstance(StringData* pData)
{
	const int32 nWantedLength = pData->GetLength();

	ASSERT(nWantedLength >= 0);

	///////////////////////////////////////////////////////
	// Number of uint64 to allocate
	// 1 is the two int32 (length and ref)
	const int32 nTotalAlloc = 1 + GetAllocatedChunkCount(nWantedLength);

	uint64* pAlloc = (uint64*) Memory::Alloc64(nTotalAlloc * 8);
	ASSERT(pAlloc); // Raise an exception ?

	const uint64* const pSrc = (uint64*) pData;

	const int32 nTotalUsed = 1 + GetUsedChunkCount(nWantedLength);
	Memory::Copy64(pAlloc, pSrc, nTotalUsed);
		
	StringData* pInst = (StringData*) pAlloc;
	pInst->m_nRef = 1;

	return pInst->m_szBuffer;
}
//////////////////////////////////////////////////////////////////////
