///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_SURFACE32_H
#define CHUSTD_SURFACE32_H

#include "ReadOnlySurface32.h"

namespace chustd {

class ReadOnlySurface8;
class String;
class Font;

enum BlendAlphaMode
{
	bamUseDstAlpha,
	bamUseSrcAlpha,
	bamUseMax
};

enum Alignment
{
	alTop = 0x10,
	alMiddle = 0x20,
	alBottom = 0x40,
	alLeft = 0x01,
	alCenter = 0x02,
	alRight = 0x04,

	alTopLeft = alTop | alLeft,
	alTopCenter = alTop | alCenter,
	alTopRight = alTop | alRight,
	alMiddleLeft = alMiddle | alLeft,
	alMiddleCenter = alMiddle | alCenter,
	alMiddleRight = alMiddle | alRight,
	alBottomLeft = alBottom | alLeft,
	alBottomCenter = alBottom | alCenter,
	alBottomRight = alBottom | alRight,

	alNoBreak = 0x100
};

class Surface32 : public ReadOnlySurface32
{
public:
	Color32* GetBuffer() const;

	void Fill(Color32 col) const;
	void Clear(Color32 col) const;

	void FillRect(const Rectangle& rect, Color32 col) const;
	void FillRect(int32 nX, int32 nY, int32 nWidth, int32 nHeight, Color32 col) const;
	
	void ClearRect(const Rectangle& rect, Color32 col) const;
	void ClearRect(int32 nX, int32 nY, int32 nWidth, int32 nHeight, Color32 col) const;

	void FillRoundRect(const Rectangle& rect, int32 nRadius, Color32 col) const;
	void FillRoundRectAA(const Rectangle& rect, int32 nRadius, Color32 col) const;

	void FillVerticalGradient(Color32 col1, Color32 col2) const;
	void FillVerticalGradient(const Rectangle& rect, Color32 col1, Color32 col2) const;
	void FillHorizontalGradient(Color32 col1, Color32 col2) const;
	void FillHorizontalGradient(const Rectangle& rect, Color32 col1, Color32 col2) const;

	void SetBlendAlphaMode(BlendAlphaMode eMode);
	void BlendSurface(const ReadOnlySurface8& surfSrc, int32 nDstX, int32 nDstY, Color32 col) const;
	void BlendSurface(const ReadOnlySurface32& surfSrc, int32 nDstX, int32 nDstY) const;

	void CopySurface(const ReadOnlySurface32& surfSrc, int32 nDstX, int32 nDstY) const;
	
	void MaskSurface(const ReadOnlySurface32& surfSrc, int32 nDstX, int32 nDstY) const;

	void DrawString(const chustd::String& str, int32 nX, int32 nY, Color32 col, const chustd::Font& font) const;
	void DrawString(const chustd::String& str, const chustd::Rectangle& rect, Alignment eAlign, Color32 col, const chustd::Font& font) const;
	void DrawLine(Point pt1, Point pt2, Color32 col) const;

	void DrawRect(const Rectangle& rect, Color32 col) const;
	
	void DrawRoundRect(const Rectangle& rect, int32 nRadius, Color32 col) const;
	void DrawRoundRectAA(const Rectangle& rect, int32 nRadius, Color32 col) const;

	enum CornerType
	{
		ctTopLeft, ctTopRight, ctBottomRight, ctBottomLeft,
		ctFilled = 0x80,
		ctTopLeftFilled = ctTopLeft | ctFilled,
		ctTopRightFilled = ctTopRight | ctFilled, 
		ctBottomRightFilled = ctBottomRight | ctFilled, 
		ctBottomLeftFilled = ctBottomLeft | ctFilled
	};
	void DrawRoundCorner(int32 nCornerX, int32 nCornerY, int32 nRadius, CornerType eType, Color32 col) const;
	void DrawRoundCornerAA(int32 nCornerX, int32 nCornerY, int32 nRadius, CornerType eType, Color32 col) const;

	void DrawPixel(int32 nX, int32 nY, Color32 col) const;
	void DrawHrzLine(int32 nX1, int32 nX2, int32 nY, Color32 col) const;
	void DrawVrtLine(int32 nX, int32 nY1, int32 nY2, Color32 col) const;
	
	Surface32();
};

} // namespace chustd

#endif // ndef CHUSTD_SURFACE32_H
