///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#ifndef CHUSTD_SURFACEBASE_H
#define CHUSTD_SURFACEBASE_H

#include "Rectangle.h"
#include "SurfaceBaseImpl.h"

namespace chustd {

class SurfaceBase  
{
public:
	int32 GetWidth() const;
	int32 GetHeight() const;
	int32 GetPitch() const;
	void* GetBuffer() const;

	Rectangle GetRectangle() const;

	bool IsValid() const { return ( GetBuffer() != null ); }

	void SetDstOffset(int32 nX, int32 nY);

	void Empty();

	SurfaceBase();
	virtual ~SurfaceBase();

protected:
	// Dfinit un buffer issu d'un morceau d'une autre surface
	// Clipping & vrification diverses
	bool SetBufferHelper(const SurfaceBase& surfSource, const Rectangle& rect);

	// Dfinit un buffer
	// Aucune vrification effectue
	bool SetBuffer(void* pBuffer, int32 nWidth, int32 nHeight, int32 nPitch, int8 nBitsPerPixel);
	bool Create(int32 nWidth, int32 nHeight, int32 nPitch, int8 nBitsPerPixel);
	void FreeBuffer();

	bool ClipSurface(const SurfaceBase& surfSrc, int32& nDstX, int32& nDstY, class SurfaceBaseImpl& sbiOut) const;
	
	SurfaceBase(class SurfaceBaseImpl* pImpl);

	void DoOffset(int32& nX, int32& nY) const;
	void DoOffset(chustd::Point& pt) const;
	void DoOffset(chustd::Rectangle& rect) const;

protected:
	SurfaceBaseImpl* m_pImpl;
};

} // namespace chustd

#endif // ndef CHUSTD_SURFACEBASE_H
