///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "TcpServer.h"
#include "Memory.h"

using namespace chustd;

bool TcpServer::Start(uint16 nPort)
{
	// Create
	if( !m_sock.Open(Socket::typeTcp) )
	{
		return false;
	}

	// Bind
	struct sockaddr_in sai;
	sai.sin_addr.s_addr = htonl(INADDR_ANY);
	sai.sin_family = AF_INET; // Network family
	sai.sin_port = htons(nPort);
	
	Memory::Zero(sai.sin_zero, 8);

	const int nBindRet = bind( SOCKET(m_sock.m_nIDSocket), (struct sockaddr*)&sai, sizeof(sai));
	if( nBindRet != 0 )
	{
		return false;
	}

	// Listen
	const int nRet = listen( SOCKET(m_sock.m_nIDSocket), 5);
	if( nRet != 0 )
	{
		return false;
	}

	return true;
}

bool TcpServer::Accept(Socket& sock)
{
	// Close the socket if the socket object was previsously used
	sock.Close();

	struct sockaddr_in sai_client;

	// "unsigned int" on Linux/G++, "int" on Windows/VC++
	socklen_t nSaiLength = sizeof(sai_client); 

	const SOCKET_ID nIDSAccepted = accept( SOCKET_ID(m_sock.m_nIDSocket),
		(struct sockaddr*) &sai_client, &nSaiLength);

	if( nIDSAccepted < 0 || nIDSAccepted == INVALID_SOCKET )
	{
		return false;
	}

	// Make temporary friend
	sock.m_nIDSocket = nIDSAccepted;
	sock.m_nType = Socket::typeTcp;
	return true;
}

void TcpServer::Stop()
{
	if( m_sock.GetSocketID() == INVALID_SOCKET )
	{
		return;
	}
	m_sock.Close();
}