///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "UdpSocket.h"
#include "Memory.h"

using namespace chustd;

//////////////////////////////////////////////////////////////////////

int32 UdpSocket::SendTo(void* pBuffer, int32 nBufferSize, int32 nFlags, uint32 nIpAddress, uint16 nPort)
{
	if( !IsOpen())
	{
		if( !super::Open(typeUdp))
			return -1;
	}

#ifdef WIN32
	struct sockaddr_in saiTo;
	const int32 nSaiSize = sizeof(saiTo);

	Memory::Zero(&saiTo, nSaiSize);

	saiTo.sin_addr.s_addr = htonl(nIpAddress);
	saiTo.sin_family = AF_INET;
	saiTo.sin_port = htons(nPort);
	
	const int64 nSocketId = GetSocketID();
	return sendto( SOCKET_ID(nSocketId), (const char*) pBuffer, nBufferSize, nFlags, (struct sockaddr*) &saiTo, nSaiSize);
#elif CHUOS
	return 0;
#endif
}

int32 UdpSocket::ReceiveFrom(void* pBuffer, int32 nBufferSize, int32 nFlags, uint32& nIpAddress, uint16& nPort)
{
	if( !IsOpen())
	{
		if( !super::Open(typeUdp))
			return -1;
	}

#ifdef WIN32
	struct sockaddr_in saiFrom;
	const int32 nSaiSize = sizeof(saiFrom);

	Memory::Zero(&saiFrom, nSaiSize);

	const int64 nSocketId = GetSocketID();
	int nFromSize = nSaiSize;
	int32 nRead = recvfrom( SOCKET_ID(nSocketId), (char*) pBuffer, nBufferSize, nFlags, (struct sockaddr*) &saiFrom, &nFromSize);

	nIpAddress = ntohl(saiFrom.sin_addr.s_addr);
	nPort = ntohs(saiFrom.sin_port);
	return nRead;
#elif CHUOS
	return 0;
#endif
}
