///////////////////////////////////////////////////////////////////////////////
// This file is part of the chustd library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chustd.h
///////////////////////////////////////////////////////////////////////////////

#include "Master.h"
#include "ZStream.h"
#include "Memory.h"

#include "File.h"
#include "StringBuilder.h"
#include "Console.h"

// inflate / deflate (PNG)
#include "zlib/zlib.h"
#include  "zlib/deflate.h"
//////////////////////////////////////////////////////////////////////
namespace chustd {
class ZStreamImpl
{
public:
	z_stream m_ZLibStream;
};
}

//////////////////////////////////////////////////////////////////////
using namespace chustd;
//////////////////////////////////////////////////////////////////////
#if 0
struct z_deflate_state
{
    z_streamp strm;
    int   status;        /* as the name implies */
    Bytef *pending_buf;  /* output still pending */
    uint32   pending_buf_size; /* size of pending_buf */
    Bytef *pending_out;  /* next pending byte to output to the stream */
    uInt   pending;      /* nb of bytes in the pending buffer */
    int   wrap;          /* bit 0 true for zlib, bit 1 true for gzip */
    gz_headerp  gzhead;  /* gzip header information to write */
    uInt   gzindex;      /* where in extra, name, or comment */
    Byte  method;        /* STORED (for zip only) or DEFLATED */
    int   last_flush;    /* value of flush param for previous deflate call */

                /* used by deflate.c: */

    uInt  w_size;        /* LZ77 window size (32K by default) */
    uInt  w_bits;        /* log2(w_size)  (8..16) */
    uInt  w_mask;        /* w_size - 1 */

    Bytef *window;

    uint32 window_size;

    Posf *prev;

    Posf *head; /* Heads of the hash chains or NIL. */

    uInt  ins_h;          /* hash index of string to be inserted */
    uInt  hash_size;      /* number of elements in hash table */
    uInt  hash_bits;      /* log2(hash_size) */
    uInt  hash_mask;      /* hash_size-1 */

    uInt  hash_shift;

    long block_start;

    uInt match_length;           /* length of best match */
    IPos prev_match;             /* previous match */
    int match_available;         /* set if previous match exists */
    uInt strstart;               /* start of string to insert */
    uInt match_start;            /* start of matching string */
    uInt lookahead;              /* number of valid bytes ahead in window */

    uInt prev_length;

    uInt max_chain_length;

    uInt max_lazy_match;
#   define max_insert_length  max_lazy_match

    int level;    /* compression level (1..9) */
    int strategy; /* favor or force Huffman coding*/

    uInt good_match;

    int nice_match; /* Stop searching when current match exceeds this */

                /* used by trees.c: */
    /* Didn't use ct_data typedef below to supress compiler warning */
    struct z_ct_data_s dyn_ltree[HEAP_SIZE];   /* literal and length tree */
    struct z_ct_data_s dyn_dtree[2*D_CODES+1]; /* distance tree */
    struct z_ct_data_s bl_tree[2*BL_CODES+1];  /* Huffman tree for bit lengths */

    struct z_tree_desc_s l_desc;               /* desc. for literal tree */
    struct z_tree_desc_s d_desc;               /* desc. for distance tree */
    struct z_tree_desc_s bl_desc;              /* desc. for bit length tree */

    ush bl_count[MAX_BITS+1];
    /* number of codes at each bit length for an optimal tree */

    int heap[2*L_CODES+1];      /* heap used to build the Huffman trees */
    int heap_len;               /* number of elements in the heap */
    int heap_max;               /* element of largest frequency */
    /* The sons of heap[n] are heap[2*n] and heap[2*n+1]. heap[0] is not used.
     * The same heap array is used to build all trees.
     */

    uch depth[2*L_CODES+1];
    /* Depth of each subtree used as tie breaker for trees of equal frequency
     */

    uchf *l_buf;          /* buffer for literals or lengths */

    uInt  lit_bufsize;

    uInt last_lit;      /* running index in l_buf */

    ushf *d_buf;

    uint32 opt_len;        /* bit length of current block with optimal trees */
    uint32 static_len;     /* bit length of current block with static trees */
    uInt matches;       /* number of string matches in current block */
    int last_eob_len;   /* bit length of EOB code for last block */

#ifdef DEBUG
    uint32 compressed_len; /* total bit length of compressed file mod 2^32 */
    uint32 bits_sent;      /* bit length of compressed data sent mod 2^32 */
#endif

    ush bi_buf;
    int bi_valid;
};
#endif

#define HOP(zoum) str = L#zoum; sb += str.Pad(20, ' '); sb += L": "; sb += String::FromInt( int32(int64(zoum)), 'x'); sb += L"\n";

void ZStream::DebugDump()
{
	z_stream& zs = m_pImpl->m_ZLibStream;
	
	StringBuilder sb;
	String str;

	HOP(zs.next_in);
	HOP(zs.avail_in);
	HOP(zs.total_in);
	HOP(zs.next_out);
	HOP(zs.avail_out);
	HOP(zs.total_out);
	HOP(zs.msg);
	HOP(zs.state);
	HOP(zs.zalloc);
	HOP(zs.zfree);
	HOP(zs.opaque);
	HOP(zs.data_type);
	HOP(zs.adler);
	HOP(zs.reserved);

	z_deflate_state* pState = zs.state;
	sb += L"--- State ---\n";

    HOP(pState->strm);
    HOP(pState->status);
    HOP(pState->pending_buf);
    HOP(pState->pending_buf_size);
    HOP(pState->pending_out);
    HOP(pState->pending);
    HOP(pState->wrap);
    HOP(pState->gzhead);
    HOP(pState->gzindex);
    HOP(pState->method);
    HOP(pState->last_flush);
    HOP(pState->w_size);
    HOP(pState->w_bits);
    HOP(pState->w_mask);
    HOP(pState->window);
    HOP(pState->window_size);
    HOP(pState->prev);
    HOP(pState->head); 
    HOP(pState->ins_h);          
    HOP(pState->hash_size);      
    HOP(pState->hash_bits);      
    HOP(pState->hash_mask);      
    HOP(pState->hash_shift);
    HOP(pState->block_start);
    HOP(pState->match_length);           
    HOP(pState->prev_match);             
    HOP(pState->match_available);         
    HOP(pState->strstart);               
    HOP(pState->match_start);            
    HOP(pState->lookahead);              
    HOP(pState->prev_length);
    HOP(pState->max_chain_length);
    HOP(pState->max_lazy_match);
    HOP(pState->level);    
    HOP(pState->strategy); 
    HOP(pState->good_match);
    HOP(pState->nice_match); 

	/*
    struct z_ct_data_s dyn_ltree[HEAP_SIZE];
    struct z_ct_data_s dyn_dtree[2*D_CODES+1];
    struct z_ct_data_s bl_tree[2*BL_CODES+1];

    struct z_tree_desc_s l_desc;
    struct z_tree_desc_s d_desc;
    struct z_tree_desc_s bl_desc;

    ush bl_count[MAX_BITS+1];

    int heap[2*L_CODES+1];
    int heap_len;
    int heap_max;
    uch depth[2*L_CODES+1];
	*/

    HOP(pState->l_buf);          
    HOP(pState->lit_bufsize);
    HOP(pState->last_lit);      
    HOP(pState->d_buf);
    HOP(pState->opt_len);
    HOP(pState->static_len);     
    HOP(pState->matches);       
    HOP(pState->last_eob_len);   
    HOP(pState->bi_buf);
    HOP(pState->bi_valid);
		
	Console::Write(sb.ToString());
}

ZStream::ZStream()
{
	m_pImpl = new ZStreamImpl;
	
	Memory::Zero(&m_pImpl->m_ZLibStream, sizeof(m_pImpl->m_ZLibStream));
}

ZStream::~ZStream()
{
	delete m_pImpl;
}

void ZStream::SetBuffers(const uint8* pInNext, uint32 nInAvailable, uint8* pOutNext, uint32 nOutAvailable)
{
	m_pImpl->m_ZLibStream.next_in = (unsigned char*) pInNext;
	m_pImpl->m_ZLibStream.avail_in = nInAvailable;
	
	m_pImpl->m_ZLibStream.next_out = (unsigned char*) pOutNext;
	m_pImpl->m_ZLibStream.avail_out = nOutAvailable;
}

uint32 ZStream::GetOutAvailable() const
{
	return m_pImpl->m_ZLibStream.avail_out;
}

uint32 ZStream::GetOutTotalRead() const
{
	return m_pImpl->m_ZLibStream.total_out;
}

///////////////////////
// Compression

int ZStream::DeflateInit(int nLevel)
{
	return deflateInit(&m_pImpl->m_ZLibStream, nLevel);
}
int ZStream::DeflateInit2(int nLevel, uint8 nMethod, int nWindowBits, int nMemLevel, int nStrategy)
{
	return deflateInit2(&m_pImpl->m_ZLibStream, nLevel, nMethod, nWindowBits, nMemLevel, nStrategy);
}
int ZStream::Deflate(int flush)
{
	return deflate(&m_pImpl->m_ZLibStream, flush);
}
int ZStream::DeflateParams(int nNewLevel, int nStrategy)
{
	return deflateParams(&m_pImpl->m_ZLibStream, nNewLevel, nStrategy);
}
int ZStream::DeflateEnd()
{
	return deflateEnd(&m_pImpl->m_ZLibStream);
}

int ZStream::DeflateSetDictionary(const uint8* pDictionary, uint32 nDictLength)
{
	return deflateSetDictionary(&m_pImpl->m_ZLibStream, (const unsigned char*) pDictionary, nDictLength);
}
int ZStream::DeflateReset()
{
	return deflateReset(&m_pImpl->m_ZLibStream);
}

uint32 ZStream::DeflateBound(uint32 nSourceLength)
{
	return deflateBound(0, nSourceLength);
}
	
///////////////////////
// Uncompression
int ZStream::InflateInit()
{
	return inflateInit(&m_pImpl->m_ZLibStream);
}
int ZStream::InflateInit2(int nWindowBits)
{
	return inflateInit2(&m_pImpl->m_ZLibStream, nWindowBits);
}

int ZStream::Inflate(int nFlushType)
{
	return inflate(&m_pImpl->m_ZLibStream, nFlushType);
}
int ZStream::InflateEnd()
{
	return inflateEnd(&m_pImpl->m_ZLibStream);
}

int ZStream::InflateSetDictionary(const uint8* pDictionary, uint32 nDictLength)
{
	return inflateSetDictionary(&m_pImpl->m_ZLibStream, (const unsigned char*) pDictionary, nDictLength);
}
int ZStream::InflateSync()
{
	return inflateSync(&m_pImpl->m_ZLibStream);
}
int ZStream::InflateReset()
{
	return inflateReset(&m_pImpl->m_ZLibStream);
}
int ZStream::InflateSyncPoint()
{
	return inflateSyncPoint(&m_pImpl->m_ZLibStream);
}

// Compresses the source buffer into the destination buffer.  sourceLen is
// the byte length of the source buffer. Upon entry, destLen is the total
// size of the destination buffer, which must be at least the value returned
// by compressBound(sourceLen). Upon exit, destLen is the actual size of the
// compressed buffer.
//
// This function can be used to compress a whole file at once if the
// input file is mmap'ed.
//
// compress returns Z_OK if success, Z_MEM_ERROR if there was not
// enough memory, Z_BUF_ERROR if there was not enough room in the output buffer.

int ZStream::Compress(uint8* pDest, uint32* pDestLen, const uint8* pSource, uint32 nSourceLen, int nLevel)
{
	unsigned long nDestLen = *pDestLen;
	int nRet = compress2( (unsigned char*) pDest, &nDestLen, (const unsigned char*) pSource, nSourceLen, nLevel);
	
	*pDestLen = nDestLen;

	return nRet;
}

const char* ZStream::GetLastError() const
{
	if( m_pImpl == null)
	{
		return "";
	}
	
	const char* psz = m_pImpl->m_ZLibStream.msg;
	if( psz == null)
	{
		return "";
	}
	return psz;
}

