///////////////////////////////////////////////////////////////////////////////
// This file is part of the chuwin32 library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chuwin32.h
///////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "DirDlg.h"

//////////////////////////////////////////////////////////////////////
using namespace chuwin32;
//////////////////////////////////////////////////////////////////////

DirDlg::DirDlg(HWND hParentWnd/*=NULL*/)
{
	m_hParentWnd = hParentWnd;
}

DirDlg::~DirDlg()
{

}

// Permet de se placer tout de suite dans un rpertoire  l'init
// lpData contient l'ID du rpertoire
int CALLBACK DirDlg::BrowseCallbackProc(HWND hWnd, UINT uMsg,
										 LPARAM lParam, LPARAM lpData)
{
	if(uMsg == BFFM_INITIALIZED)
	{
		// wParam == FALSE => lParam == pItemIdList
		ITEMIDLIST* pidlSel = (ITEMIDLIST*) lpData;
		if( pidlSel != NULL)
		{
			::SendMessage(hWnd, BFFM_SETSELECTION, FALSE, lpData);
		}
	}
	return 0;
}

int DirDlg::DoModal()
{
	int nReturn = IDCANCEL;
	LPMALLOC pMalloc;    // Gets the Shell's default allocator
	if (::SHGetMalloc(&pMalloc) == NOERROR)
	{
		// Rcupre l'id correspondant du chemin de dpart
		ITEMIDLIST* pidlSel = GetPIDLFromPath(m_strStartDir);
	
		wchar szBuffer[MAX_PATH];

		BROWSEINFOW bi;
		bi.hwndOwner = m_hParentWnd;
		bi.pidlRoot = NULL;
		bi.pszDisplayName = szBuffer;
		bi.lpszTitle = m_strTitle.GetBuffer();
		bi.ulFlags = BIF_RETURNONLYFSDIRS;
		bi.lpfn = BrowseCallbackProc; // Fonction callback
		bi.lParam = PtrToUlong(pidlSel);    // lpDatat rcupr dans la fonction callback
		bi.iImage = 0;
	
		// The fonction
		ITEMIDLIST* pItemIDList = SHBrowseForFolderW(&bi);
	
		if( pItemIDList != NULL)
		{
			// Converion ID rpertoire slectionn vers chane de carcatres
			wchar szBuffer[MAX_PATH];
			if(::SHGetPathFromIDListW(pItemIDList, szBuffer))
			{ 
				// At this point pszBuffer contains the selected path
				// Place la chane dans le contrle d'dition
				m_strDir = szBuffer;
				nReturn = IDOK;
			}
			// Free the PIDL allocated by SHBrowseForFolder.
			pMalloc->Free(pItemIDList);
		}


		// Libre l'item de dpart
		if( pidlSel != NULL)
			pMalloc->Free(pidlSel);	

		// Release the shell's allocator.
		pMalloc->Release();
	}

	return nReturn;
}


LPITEMIDLIST DirDlg::GetPIDLFromPath(const chustd::String& strPath)
{
	LPITEMIDLIST  pidl;
	LPSHELLFOLDER pDesktopFolder;
	OLECHAR       olePath[MAX_PATH];
	ULONG         chEaten;
	ULONG         dwAttributes;
	
	// Get a pointer to the Desktop's IShellFolder interface.
	if (SUCCEEDED(SHGetDesktopFolder(&pDesktopFolder)))
	{
		// IShellFolder::ParseDisplayName requires the file name be in Unicode.
		int32 nByteCount = chustd::Math::Max(MAX_PATH, (strPath.GetLength() + 1 )* 2);
		chustd::Memory::Copy16(olePath, strPath.GetBuffer(), nByteCount);

		// Convert the path to an ITEMIDLIST.
		HRESULT hr = pDesktopFolder->ParseDisplayName(
			NULL,
			NULL,
			olePath,
			&chEaten,
			&pidl,
			&dwAttributes);
		if (FAILED(hr))
		{
			// Handle error
			return NULL;
		}
		
		// pidl now contains a pointer to an ITEMIDLIST for .\readme.txt.
		// This ITEMIDLIST needs to be freed using the IMalloc allocator
		// returned from SHGetMalloc()
	
		// Release the desktop folder object
		pDesktopFolder->Release();

		return pidl;
	}
	return NULL;
}

void DirDlg::SetTitle(const chustd::String& strTitle)
{
	m_strTitle = strTitle;
}

void DirDlg::SetStartDir(const chustd::String& strStartDir)
{
	m_strStartDir = strStartDir;

	if( m_strStartDir == L"\\" || m_strStartDir == L"/")
	{
		// Si le rpertoire initial vaut "/" ou "\" alors on change
		// sa valeur pour faire correspondre  une lettre de lecteur

		wchar szCurDir[MAX_PATH];
		GetCurrentDirectoryW(sizeof(szCurDir), szCurDir);

		m_strStartDir = chustd::File::GetDrive(szCurDir);
		m_strStartDir = m_strStartDir + L"\\";
	}
}

const chustd::String& DirDlg::GetDir() const
{
	return m_strDir;
}
