///////////////////////////////////////////////////////////////////////////////
// This file is part of the chuwin32 library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chuwin32.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TraceCtl.h"
#include "gui.h"

//////////////////////////////////////////////////////////////////////
using namespace chuwin32;
//////////////////////////////////////////////////////////////////////
#ifndef IDC_HAND
#define IDC_HAND            MAKEINTRESOURCE(32649)
#endif
//////////////////////////////////////////////////////////////////////
uint32 TraceCtl::ms_nWmAddText = 0;
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
TraceCtl::TextPiece* TraceCtl::Line::AddOrReusePiece()
{
	TextPiece* pResult = null;

	const int nPieceCount = m_aPieces.GetSize();
	if( nPieceCount == 0 )
	{
		// Add one
		if( !m_aPieces.SetSize(1) )
		{
			return null;
		}
	}
	else
	{
		const TextPiece& lastpiece = m_aPieces.GetLast();
		if( lastpiece.m_str.IsEmpty() )
		{
			// Reuse the last piece
		}
		else
		{
			// Add a new piece
			if( m_aPieces.Add() < 0 )
			{
				return null;
			}
		}
	}
	return &(m_aPieces.GetLast());
}

//////////////////////////////////////////////////////////////////////
TraceCtl::TraceCtl()
{
	m_hWnd = NULL;
	m_hFont = NULL;
	m_hIconFont = NULL;
	m_nMarginLeft = 0;
	m_bDragStart = false;
	m_nLargestLineWidth = 0;
	m_nLineHeight = 0;
	m_nAvgCharWidth = 0;
}

TraceCtl::~TraceCtl()
{
	DeleteObject(m_hIconFont);
}

bool TraceCtl::Create(int x, int y, int nWidth, int nHeight, HWND hParentWnd, int nID)
{
	/////////////////////////////////////////////////////////////
	// Enregistre la classe de fentre
	static const wchar szClassName[] = L"chuwin32 TraceCtl Window Class";
	
	HINSTANCE hInstance = GetModuleHandle(NULL);

	WNDCLASSEXW wcex;
	wcex.cbSize			= sizeof(WNDCLASSEX); 
	wcex.style			= CS_OWNDC | CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wcex.lpfnWndProc	= (WNDPROC)WndProcStatic;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szClassName;
	wcex.hIconSm		= NULL;

	ATOM atom = RegisterClassExW(&wcex);
	if( atom == 0)
	{
		return false;
	}

	int nStyle = WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_HSCROLL;
	int nExStyle = WS_EX_TRANSPARENT | WS_EX_STATICEDGE;
	
	static const wchar szWindowName[] = L"TraceCtl";

	if( !CreateEx(nExStyle, szClassName, szWindowName, nStyle, x, y, nWidth, nHeight, hParentWnd, nID))
	{
		return false;
	}

	// Create the window message involved in writting text
	if( ms_nWmAddText == 0 )
	{
		ms_nWmAddText = RegisterWindowMessageW(NULL);
	}

	//////////////////////////////////////////////////
	// Create the font
	LOGFONT lf;
	SystemParametersInfo(SPI_GETICONTITLELOGFONT, sizeof(LOGFONT), &lf, 0);
	m_hIconFont = CreateFontIndirect(&lf);
	if( m_hIconFont == NULL)
	{
		SetFont( (HFONT) GetStockObject(DEFAULT_GUI_FONT));
	}
	else
	{
		SetFont(m_hIconFont);
	}
	
	InitScrollSizeVert();
	InitScrollSizeHorz();

	return true;
}

LRESULT TraceCtl::WndProc(UINT nMsg, WPARAM wParam, LPARAM lParam)
{
	if( nMsg == TraceCtl::ms_nWmAddText )
	{
		return OnAddText(wParam, lParam);
	}

	switch(nMsg)
	{
	case WM_PAINT:
		return OnPaint(wParam, lParam);
		
	case WM_ERASEBKGND:
		return OnEraseBkgnd(wParam, lParam);

	case WM_VSCROLL:
		return OnVScroll(wParam, lParam);

	case WM_HSCROLL:
		return OnHScroll(wParam, lParam);

	case WM_SIZE:
		return OnSize(wParam, lParam);

	case WM_RBUTTONDOWN:
		return OnRButtonDown(wParam, lParam);

	case WM_LBUTTONDOWN:
		return OnLButtonDown(wParam, lParam);

	case WM_LBUTTONUP:
		return OnLButtonUp(wParam, lParam);

	case WM_MOUSEMOVE:
		return OnMouseMove(wParam, lParam);

	case WM_LBUTTONDBLCLK:
		return OnLButtonDblClk(wParam, lParam);

	case WM_KEYDOWN:
		return OnKeyDown(wParam, lParam);
	
	case WM_MOUSEWHEEL:
		return OnMouseWheel(wParam, lParam);

	case WM_DESTROY:
		return OnDestroy(wParam, lParam);

	default:
		return DefWindowProc(m_hWnd, nMsg, wParam, lParam);
	}
	return 0;
}

int TraceCtl::OnKeyDown(WPARAM wParam, LPARAM lParam)
{
	if( wParam == VK_DOWN)
	{
		OnVScroll(SB_LINEDOWN, LPARAM(m_hWnd));
	}
	else if( wParam == VK_UP)
	{
		OnVScroll(SB_LINEUP, LPARAM(m_hWnd));
	}
	else if( wParam == VK_LEFT)
	{
		OnHScroll(SB_LINEUP, LPARAM(m_hWnd));
	}
	else if( wParam == VK_RIGHT)
	{
		OnHScroll(SB_LINEDOWN, LPARAM(m_hWnd));
	}
	else if( wParam == VK_NEXT)
	{
		OnVScroll(SB_PAGEDOWN, LPARAM(m_hWnd));
	}
	else if( wParam == VK_PRIOR)
	{
		OnVScroll(SB_PAGEUP, LPARAM(m_hWnd));
	}
	else if( wParam == VK_HOME)
	{
		if( GetAsyncKeyState(VK_CONTROL) != 0)
		{
			OnVScroll(SB_TOP, LPARAM(m_hWnd));
		}
		else
		{
			OnHScroll(SB_TOP, LPARAM(m_hWnd));
		}
	}
	else if( wParam == VK_END)
	{
		if( GetAsyncKeyState(VK_CONTROL) != 0)
		{
			OnVScroll(SB_BOTTOM, LPARAM(m_hWnd));
		}
		else
		{
			OnHScroll(SB_BOTTOM, LPARAM(m_hWnd));
		}
	}

	return 0;
}

int TraceCtl::OnMouseWheel(WPARAM wParam, LPARAM lParam)
{
	int zDelta = GET_WHEEL_DELTA_WPARAM(wParam);

	int32 nPos = GetNewScrollPos32(SB_VERT);
	nPos -= (zDelta * m_nLineHeight) / 120;
	SetScrollPos(SB_VERT, nPos, TRUE); // TRUE bRedraw
	OnVScroll(SB_THUMBTRACK, LPARAM(m_hWnd));

	return 0;
}

TraceCtl::TextPiece* TraceCtl::GetLinkFromPoint(int x, int y, RECT* pLocalRect)
{
	uint32 nScrollStart = GetNewScrollPos32(SB_VERT);
	uint32 nLinePixY = nScrollStart + y;
	int32 nLine = nLinePixY / m_nLineHeight;
	
	// Ligne inexistante ?
	if( nLine >= m_aLines.GetSize())
		return 0;

	Line& line = m_aLines[nLine];

	// Parcours des morceaux de texte
	HDC hResDC = GetDC();
	SelectObject(hResDC, m_hFont);

	TextPiece* pResult = null;

	int32 nX1 = m_nMarginLeft;
	int32 nX2 = 0;
	foreach(line.m_aPieces, iPiece)
	{
		TextPiece& tp = line.m_aPieces[iPiece];
		
		SIZE size;
		GetTextExtentPoint32W(hResDC, tp.m_str.GetBuffer(), tp.m_str.GetLength(), &size);

		nX2 = nX1 + size.cx;

		if( nX1 <= x && x < nX2)
		{
			// Dedans !
			if( !tp.m_strUrl.IsEmpty())
			{
				// Un lien !
				pResult = &tp;

				break;
			}
		}
		nX1 = nX2;
	}
	
	ReleaseDC(hResDC);

	return pResult;
}

int TraceCtl::OnLinkDragBegin(const chustd::String& strUrl)
{
	return dragNone;
}

void TraceCtl::OnLinkDoubleClick(const chustd::String& strUrl)
{

}

int TraceCtl::OnLButtonDown(WPARAM wParam, LPARAM lParam)
{
	// On va chercher si on clique sur un lien
	int x = GET_X_LPARAM(lParam); // Inclure <windowsx.h> dans stdafx.h
	int y = GET_Y_LPARAM(lParam); 

	TextPiece* pTP = GetLinkFromPoint(x, y);
	if( pTP != null)
	{
		// Transforme le curseur
		HCURSOR hCurs = LoadCursor(NULL, MAKEINTRESOURCE(IDC_HAND));
		SetCursor(hCurs);

		m_bDragStart = true;
		m_ptDrag.x = x;
		m_ptDrag.y = y;
		m_pDragText = pTP;
	}

	return 0;
}

int TraceCtl::OnLButtonUp(WPARAM wParam, LPARAM lParam)
{
	// On va chercher si on clique sur un lien
	int x = GET_X_LPARAM(lParam); // Inclure <windowsx.h> dans stdafx.h
	int y = GET_Y_LPARAM(lParam);

	m_bDragStart = false;
	m_pDragText = null;

	TextPiece* pTP = GetLinkFromPoint(x, y);
	if( pTP != null)
	{
		// Transforme le curseur
		HCURSOR hCurs = LoadCursor(NULL, MAKEINTRESOURCE(IDC_HAND));
		SetCursor(hCurs);
	}

	return 0;
}

int TraceCtl::OnLButtonDblClk(WPARAM wParam, LPARAM lParam)
{
	int x = GET_X_LPARAM(lParam); // Inclure <windowsx.h> dans stdafx.h
	int y = GET_Y_LPARAM(lParam);

	TextPiece* pTP = GetLinkFromPoint(x, y);
	if( pTP != null)
	{
		// Transforme le curseur
		HCURSOR hCurs = LoadCursor(NULL, MAKEINTRESOURCE(IDC_HAND));
		SetCursor(hCurs);

		OnLinkDoubleClick(pTP->m_strUrl);
	}
	return 0;
}

int TraceCtl::OnMouseMove(WPARAM wParam, LPARAM lParam)
{
	// On va chercher si on clique sur un lien
	int x = GET_X_LPARAM(lParam); // Inclure <windowsx.h> dans stdafx.h
	int y = GET_Y_LPARAM(lParam); 

	if( (wParam & MK_LBUTTON) && m_bDragStart)
	{
		int nDeltaX = chustd::Math::Abs( int32(m_ptDrag.x - x));
		int nDeltaY = chustd::Math::Abs( int32(m_ptDrag.y - y));

		if( nDeltaX > 4 || nDeltaY > 4)
		{
			COLORREF crPrevious = m_pDragText->m_cr;
			m_pDragText->m_cr = RGB(127, 127, 255);
			
			HDC hDC = GetDC();
			Draw(hDC);
			ReleaseDC(hDC);

			int nDragResult = OnLinkDragBegin(m_pDragText->m_strUrl);
			
			m_bDragStart = false;
			m_pDragText->m_cr = crPrevious;

			if( nDragResult == dragMove)
			{
				// Le lien n'est plus valide
				m_pDragText->m_cr = RGB(20, 20, 30);
				m_pDragText->m_strUrl.Empty();
			}
			else
			{
				m_pDragText->m_cr = crPrevious;
			}

			hDC = GetDC();
			Draw(hDC);
			ReleaseDC(hDC);
		}
	}
	else
	{
		m_bDragStart = false;
	}

	TextPiece* pTP = GetLinkFromPoint(x, y);
	if( pTP != null)
	{
		// Transforme le curseur
		HCURSOR hCurs = LoadCursor(NULL, MAKEINTRESOURCE(IDC_HAND));
		SetCursor(hCurs);
	}
	return 0;
}

int TraceCtl::OnRButtonDown(WPARAM wParam, LPARAM lParam)
{
	int nControlID = 1000;
	NMHDR nm;
	nm.code = NM_RCLICK; // Inclure <commctrl.h> dans stdafx.h
	nm.hwndFrom = m_hWnd;
	nm.idFrom = nControlID;

	SendMessage( GetParent(), WM_NOTIFY, nControlID, (LPARAM) &nm);
	return 0;
}

void TraceCtl::Draw(HDC hdcWindow)
{
	// One thread at a time can draw
	chustd::TmpLock tmplock(m_cs);

	RECT rectClient = GetClientRect();
	const int nClientHeight = rectClient.bottom - rectClient.top;
	const int nClientWidth = rectClient.right - rectClient.left;
	
	///////////////////////////////////////////////

	DeviceContext dcDib(m_dib.m_hDC);

	COLORREF colBk = GetSysColor(COLOR_WINDOW);
	dcDib.SelectObject(m_hFont);
	
	// For correct anti-aliasing, do not render any background when drawing the font,
	// we will draw the whole background ourselves
	dcDib.SetBkMode(TRANSPARENT);

	const int nStartX = -GetScrollPos(SB_HORZ);
	
	int y = 0;

	int32 nPos = GetScrollPos(SB_VERT);

	y -= nPos;

	// Regarde  partir de quel indice il faut commencer  dessiner
	int nIndexStart = 0;
	int nHeightBefore = -y;
	if( nHeightBefore > 0)
	{
		nIndexStart = nHeightBefore / m_nLineHeight;
		const int nRndHeight = nIndexStart * m_nLineHeight;
		y += nRndHeight;
	}

	const int nLineCount = m_aLines.GetSize();

	// m_nLineHeight peut tre  0  l'initialisation du contrle
	if( m_nLineHeight > 0)
	{
		const int nVisibleIndexCount = (nClientHeight / m_nLineHeight) + 1;
		int nIndexEnd = nIndexStart + nVisibleIndexCount;
		if( nIndexEnd >= nLineCount)
		{
			nIndexEnd = nLineCount - 1;
		}

		for(int iLine = nIndexStart; iLine <= nIndexEnd; ++iLine)
		{
			const Line& line = m_aLines[iLine];
			
			RECT rectClearLine;
			rectClearLine.top = 0;
			rectClearLine.bottom = m_dib.GetHeight();
			rectClearLine.left = 0;
			rectClearLine.right = rectClient.right - rectClient.left;
			dcDib.FillSolidRect(rectClearLine, colBk);

			int nDibX = nStartX + m_nMarginLeft;
			int nLineWidth = m_nMarginLeft;

			const int nPieceCount = line.m_aPieces.GetSize();
			for(int iPiece = 0; iPiece < nPieceCount; ++iPiece)
			{
				const TextPiece& tp = line.m_aPieces[iPiece];

				SetBkColor(dcDib.m_hDC, RGB(iPiece * 127+100, 255, 200));
				int nWidth = DrawTextPiece(dcDib.m_hDC, nDibX, 0, tp.m_str, tp.m_cr);
				if( !tp.m_strUrl.IsEmpty())
				{
					/*
					// Underline links ?
					POINT ptOld;
					int nLineY = y + m_nLineHeight - 1;
					::MoveToEx(hResDC, x , nLineY, &ptOld);
					::LineTo(hResDC, x + nWidth, nLineY);
					*/
				}

				nDibX += nWidth;
				nLineWidth += nWidth;
			}

			// Blit the line to the window
			m_dib.Blit(hdcWindow, 0, y);

			y += m_nLineHeight;
		}
	}

	// Clear the remaining client area
	DeviceContext dcWindow(hdcWindow);
	rectClient.top = y;
	dcWindow.FillSolidRect(rectClient, colBk);
}

int TraceCtl::DrawTextPiece(HDC hDC, int x, int y, const chustd::String& str, COLORREF cr)
{
	SetTextColor(hDC, cr);
	TextOutW(hDC, x, y, str.GetBuffer(), str.GetLength());
	SIZE size;
	GetTextExtentPoint32W(hDC, str.GetBuffer(), str.GetLength(), &size);

	return size.cx;
}

int TraceCtl::OnPaint(WPARAM wParam, LPARAM lParam) 
{
	PAINTSTRUCT ps;
	HDC hPaintDC = BeginPaint(m_hWnd, &ps);

	Draw(hPaintDC);

	EndPaint(m_hWnd, &ps);
	return 0;
}

void TraceCtl::Refresh()
{
	UpdateScrollSizeVert();
	UpdateScrollSizeHorz();

	HDC hDC = GetDC();
	Draw(hDC);
	ReleaseDC(hDC);
}

///////////////////////////////////////////////////////////////////////////////////////////////////
void TraceCtl::AddText(const chustd::String& strText, COLORREF cr)
{
	AddTextInfo* pAddTextInfo = new AddTextInfo;
	if( pAddTextInfo == null )
	{
		return;
	}
	pAddTextInfo->operation = opAddText;
	pAddTextInfo->strText = strText;
	pAddTextInfo->cr = cr;

	::SendMessage(m_hWnd, ms_nWmAddText, (WPARAM) pAddTextInfo, 0);
}

void TraceCtl::AddLine(const chustd::String& strText, COLORREF cr)
{
	chustd::String strLine = strText + L"\n";
	AddText(strLine, cr);
}

void TraceCtl::AddTextAtLine(int32 nLine, const chustd::String& strText, COLORREF cr)
{
	AddTextInfo* pAddTextInfo = new AddTextInfo;
	if( pAddTextInfo == null )
	{
		return;
	}
	pAddTextInfo->operation = opAddTextAtLine;
	pAddTextInfo->strText = strText;
	pAddTextInfo->cr = cr;
	pAddTextInfo->nLine = nLine;

	::SendMessage(m_hWnd, ms_nWmAddText, (WPARAM) pAddTextInfo, 0);
}

void TraceCtl::AddLink(const chustd::String& strText, const chustd::String& strUrl)
{
	AddTextInfo* pAddTextInfo = new AddTextInfo;
	if( pAddTextInfo == null )
	{
		return;
	}
	pAddTextInfo->operation = opAddLink;
	pAddTextInfo->strText = strText;
	pAddTextInfo->strUrl = strUrl;

	::SendMessage(m_hWnd, ms_nWmAddText, (WPARAM) pAddTextInfo, 0);
}

void TraceCtl::ClearLineAt(int32 nLine)
{
	AddTextInfo* pAddTextInfo = new AddTextInfo;
	if( pAddTextInfo == null )
	{
		return;
	}
	pAddTextInfo->operation = opClearLineAt;
	pAddTextInfo->nLine = nLine;

	::SendMessage(m_hWnd, ms_nWmAddText, (WPARAM) pAddTextInfo, 0);
}

void TraceCtl::ClearLine()
{
	ClearLineAt(-1);
}

void TraceCtl::Clear()
{
	AddTextInfo* pAddTextInfo = new AddTextInfo;
	if( pAddTextInfo == null )
	{
		return;
	}
	pAddTextInfo->operation = opClearAll;

	::SendMessage(m_hWnd, ms_nWmAddText, (WPARAM) pAddTextInfo, 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////
BOOL TraceCtl::OnAddText(WPARAM wParam, LPARAM lParam)
{
	AddTextInfo* pAddTextInfo = (AddTextInfo*) wParam;
	
	// Transfert params
	Operation operation = pAddTextInfo->operation;
	chustd::String strText = pAddTextInfo->strText;
	chustd::String strUrl = pAddTextInfo->strUrl;
	COLORREF cr = pAddTextInfo->cr;
	int nLine = pAddTextInfo->nLine;

	delete pAddTextInfo;
	pAddTextInfo = null;

	switch( operation )
	{
	case opAddText:
		DoAddText(strText, cr);
		break;
	case opAddTextAtLine:
		DoAddTextAtLine(nLine, strText, cr);
		break;
	case opAddLink:
		DoAddLink(strText, strUrl);
		break;
	case opClearLineAt:
		DoClearLineAt(nLine);
		break;
	case opClearAll:
		DoClear();
		break;
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
void TraceCtl::DoAddText(const chustd::String& strText, COLORREF cr)
{
	chustd::StringArray astr = strText.SplitByEndlines();

	chustd::TmpLock tmplock(m_cs);

	if( m_aLines.GetSize() == 0)
	{
		// Il n'y a pas encore de ligne courante (le tableau est vide)
		m_aLines.SetSize(1);
	}

	// On reste sur la ligne courante
	const int nStartLine = m_aLines.GetSize() - 1;

	const int nCount = astr.GetSize();
	if( nCount > 1)
	{
		// Agrandit le tableau des lignes
		m_aLines.SetSize( m_aLines.GetSize() + nCount - 1);
	}
	else
	{

	}

	int32 nScrollStart = GetNewScrollPos32(SB_VERT);

	RECT rectToUpdate;
	::GetClientRect(m_hWnd, &rectToUpdate);
	
	rectToUpdate.top = nStartLine * m_nLineHeight - nScrollStart;
	rectToUpdate.bottom = rectToUpdate.top + nCount * m_nLineHeight;

	for(int i = 0; i < nCount; ++i)
	{
		Line& line = m_aLines[nStartLine + i];

		// Add an empty piece, except if there is already one
		TextPiece* pPiece = line.AddOrReusePiece();
		if( pPiece == null )
		{
			break;
		}

		pPiece->m_cr = cr;
		pPiece->m_str = astr[i];

		const int nWidth = ComputeLineWidth(line);
		if( nWidth > m_nLargestLineWidth)
		{
			m_nLargestLineWidth = nWidth;
		}
	}

	bool bScrollToBottom = true;

	UpdateScrollSizeVert(bScrollToBottom);
	UpdateScrollSizeHorz();
	
	InvalidateRect(m_hWnd, NULL, FALSE);
}

void TraceCtl::DoAddTextAtLine(int32 nLine, const chustd::String& strText, COLORREF cr)
{
	m_cs.Enter();

	Line& line = m_aLines[nLine];

	int32 nIndex = line.m_aPieces.Add();
	line.m_aPieces[nIndex].m_cr = cr;
	line.m_aPieces[nIndex].m_str = strText;

	const int nWidth = ComputeLineWidth(line);
	if( nWidth > m_nLargestLineWidth)
	{
		m_nLargestLineWidth = nWidth;
	}

	Refresh();

	m_cs.Leave();
}

void TraceCtl::DoAddLink(const chustd::String& strText, const chustd::String& strUrl)
{
	m_cs.Enter();

	if( m_aLines.GetSize() == 0 )
	{
		// Il n'y a pas encore de ligne courante (le tableau est vide)
		m_aLines.SetSize(1);
	}

	Line& line = m_aLines.GetLast();

	// Add an empty piece, except if there is already one
	TextPiece* pPiece = line.AddOrReusePiece();
	if( pPiece == null )
	{
		return;
	}

	pPiece->m_cr = RGB(0, 0, 255);
	pPiece->m_str = strText;
	pPiece->m_strUrl = strUrl;

	const int nWidth = ComputeLineWidth(line);
	if( nWidth > m_nLargestLineWidth)
	{
		m_nLargestLineWidth = nWidth;
	}

	Refresh();

	m_cs.Leave();
}

void TraceCtl::DoClearLineAt(int32 nLine)
{
	m_cs.Enter();
	
	if( m_aLines.GetSize() > 0)
	{
		if( nLine < 0)
		{
			Line& line = m_aLines.GetLast();
			line.m_aPieces.RemoveAll();
		}
		else
		{
			Line& line = m_aLines[nLine];
			line.m_aPieces.RemoveAll();
		}
		Refresh();
	}
	m_cs.Leave();
}

void TraceCtl::DoClear()
{
	m_cs.Enter();

	m_aLines.RemoveAll();
	m_nLargestLineWidth = 0;

	Refresh();

	m_cs.Leave();
}

///////////////////////////////////////////////////////////////////////////////////////////////////
int TraceCtl::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	m_dib.Destroy();
	return 0;
}

BOOL TraceCtl::OnEraseBkgnd(WPARAM wParam, LPARAM lParam) 
{
	return FALSE;
}

uint32 TraceCtl::GetNewScrollPos32(int nSBType, HWND hScrollBar)
{
    // First determine if the user scrolled a scroll bar control
    // on the window or scrolled the window itself
	HWND hWndScroll;
	if( hScrollBar == NULL )
		hWndScroll = m_hWnd;
	else
		hWndScroll = hScrollBar;
	
	SCROLLINFO info;
	info.cbSize = sizeof(SCROLLINFO);
	info.fMask = SIF_TRACKPOS;
	::GetScrollInfo(hWndScroll, nSBType, &info);
	
	uint32 nPos = info.nTrackPos;
	return nPos;
}

int TraceCtl::OnVScroll(WPARAM wParam, LPARAM lParam) 
{
	UINT nSBCode = LOWORD(wParam);
	HWND hScrollBar = (HWND) lParam;

	int nGetPos = GetScrollPos(SB_VERT);
	const int nBefore = nGetPos;

	int nMinPos = 0;
	int nMaxPos = 0;
	GetScrollRange(m_hWnd, SB_VERT, &nMinPos, &nMaxPos);

	switch(nSBCode)
	{
	case SB_TOP:
		nGetPos = nMinPos;
		break;
	case SB_BOTTOM:
		nGetPos = nMaxPos;
		break;
	case SB_ENDSCROLL:
		break;
	case SB_LINEDOWN:
		nGetPos += m_nLineHeight;
		break;
	case SB_LINEUP:
		nGetPos -= m_nLineHeight;
		break;
	case SB_PAGEDOWN:
		nGetPos += m_nLineHeight * 4;
		break;
	case SB_PAGEUP:
		nGetPos -= m_nLineHeight * 4;
		break;
	case SB_THUMBPOSITION:
		nGetPos = GetNewScrollPos32(SB_VERT, hScrollBar);
		break;
	case SB_THUMBTRACK:
		nGetPos = GetNewScrollPos32(SB_VERT, hScrollBar);
		break;
	}

	SetScrollPos(SB_VERT, nGetPos, TRUE); // TRUE bRedraw
	
	const int nAfter = GetScrollPos(SB_VERT);
	const int nDelta = nBefore - nAfter;
	
	InvalidateRect(m_hWnd, NULL, FALSE);
	return 0;
}

void TraceCtl::UpdateScrollSizeVert(bool bUpdatePosition /*= true*/)
{
	int nGetPos = GetScrollPos(SB_VERT);

	RECT rect = GetClientRect();
	const int nClientHeight = rect.bottom - rect.top;

	const int nLineCount = m_aLines.GetSize();

	SCROLLINFO si;
    si.cbSize = sizeof(si);
    si.fMask = SIF_PAGE | SIF_RANGE | SIF_DISABLENOSCROLL;

	if( bUpdatePosition)
	{
		si.fMask |= SIF_POS;
	}
    si.nMin = 0;
    si.nMax = nLineCount * m_nLineHeight;
    si.nPage = nClientHeight;
    si.nPos = si.nMax;
    si.nTrackPos = 0;

	SetScrollInfo(m_hWnd, SB_VERT, &si, TRUE);
}

void TraceCtl::InitScrollSizeVert()
{
	RECT rect = GetClientRect();
	const int nClientHeight = rect.bottom - rect.top;

	const int nLineCount = m_aLines.GetSize();

	SCROLLINFO si;
    si.cbSize = sizeof(si);
    si.fMask = SIF_ALL | SIF_DISABLENOSCROLL;
    si.nMin = 0;
    si.nMax = nLineCount * m_nLineHeight;
    si.nPage = nClientHeight;
    si.nPos = 0;
    si.nTrackPos = 0;

	SetScrollInfo(m_hWnd, SB_VERT, &si, TRUE);
}


int TraceCtl::OnHScroll(WPARAM wParam, LPARAM lParam) 
{
	UINT nSBCode = LOWORD(wParam);
	HWND hScrollBar = (HWND) lParam;

	int nGetPos = GetScrollPos(SB_HORZ);
	const int nBefore = nGetPos;

	int nMinPos = 0;
	int nMaxPos = 0;
	GetScrollRange(m_hWnd, SB_VERT, &nMinPos, &nMaxPos);

	switch(nSBCode)
	{
	case SB_TOP:
		nGetPos = nMinPos;
		break;
	case SB_BOTTOM:
		nGetPos = nMaxPos;
		break;
	case SB_ENDSCROLL:
		break;
	case SB_LINEDOWN:
		nGetPos += m_nAvgCharWidth;
		break;
	case SB_LINEUP:
		nGetPos -= m_nAvgCharWidth;
		break;
	case SB_PAGEDOWN:
		nGetPos += 4 * m_nAvgCharWidth;
		break;
	case SB_PAGEUP:
		nGetPos -= 4 * m_nAvgCharWidth;
		break;
	case SB_THUMBPOSITION:
		nGetPos = GetNewScrollPos32(SB_HORZ, hScrollBar);
		break;
	case SB_THUMBTRACK:
		nGetPos = GetNewScrollPos32(SB_HORZ, hScrollBar);
		break;
	}

	SetScrollPos(SB_HORZ, nGetPos, TRUE);
	
	const int nAfter = GetScrollPos(SB_HORZ);
	const int nDelta = nBefore - nAfter;
	
	InvalidateRect(m_hWnd, NULL, FALSE);
	return 0;
}


void TraceCtl::UpdateScrollSizeHorz()
{
	int nGetPos = GetScrollPos(SB_HORZ);

	RECT rect = GetClientRect();
	const int nClientWidth = rect.right - rect.left;
	
	SCROLLINFO si;
    si.cbSize = sizeof(si);
    si.fMask = SIF_PAGE | SIF_RANGE | SIF_DISABLENOSCROLL; // On ne met pas  jour la position
    si.nMin = 0;
    si.nMax = m_nLargestLineWidth;
    si.nPage = nClientWidth;
    si.nPos = si.nMax;
    si.nTrackPos = 0;

	SetScrollInfo(m_hWnd, SB_HORZ, &si, TRUE);
}

void TraceCtl::InitScrollSizeHorz()
{
	RECT rect = GetClientRect();
	const int nClientWidth = rect.right - rect.left;

	SCROLLINFO si;
    si.cbSize = sizeof(si);
    si.fMask = SIF_ALL | SIF_DISABLENOSCROLL;
    si.nMin = 0;
    si.nMax = m_nLargestLineWidth;
    si.nPage = nClientWidth;
    si.nPos = 0;
    si.nTrackPos = 0;

	SetScrollInfo(m_hWnd, SB_HORZ, &si, TRUE);
}

int TraceCtl::ComputeLineWidth(const Line& line)
{
	HDC hResDC = GetDC();
	SelectObject(hResDC, m_hFont);

	int x = m_nMarginLeft;

	const int nPieceCount = line.m_aPieces.GetSize();
	for(int iPiece = 0; iPiece < nPieceCount; ++iPiece)
	{
		const TextPiece& tp = line.m_aPieces[iPiece];
		
		const chustd::String& str = tp.m_str;
		
		SIZE size;
		GetTextExtentPoint32W(hResDC, str.GetBuffer(), str.GetLength(), &size);
		
		x += size.cx;
	}

	ReleaseDC(hResDC);
	return x;
}

int TraceCtl::OnSize(WPARAM wParam, LPARAM lParam)
{
	int nType = int(wParam);
	int cx = LOWORD(lParam);
	int cy = HIWORD(lParam);
	
	chustd::TmpLock tmplock(m_cs);

	// On recre le DIB
	UpdateScrollSizeVert(false); // Pas de mise  jour de position sinon on se retrouve tout en bas au resize
	UpdateScrollSizeHorz();
	
	HDC hDC = GetDC();
	Draw(hDC);
	ReleaseDC(hDC);

	return 0;
}

void TraceCtl::Resize(int cx, int cy)
{
	MoveWindow(m_hWnd, 0, 0, cx, cy, TRUE);
}

void TraceCtl::SetFont(const HFONT& hFont)
{
	m_cs.Enter();

	m_hFont = hFont;

	HDC hDC = GetDC();

	SelectObject(hDC, m_hFont);
	
	TEXTMETRIC tm;
	GetTextMetrics(hDC, &tm);

	m_nLineHeight = tm.tmHeight;
	m_nMarginLeft = tm.tmAveCharWidth / 2;
	m_nAvgCharWidth = tm.tmAveCharWidth;

	// Recreate the dib for off line text rendering
	int nMaxWidth = ::GetSystemMetrics(SM_CXFULLSCREEN);

	m_dib.Destroy();
	m_dib.Create(hDC, nMaxWidth, m_nLineHeight);

	ReleaseDC(hDC);

	m_cs.Leave();
}

const HFONT& TraceCtl::GetFont() const 
{
	return m_hFont;
}

void TraceCtl::ResetHrzPos()
{
	m_cs.Enter();

	if( m_aLines.GetSize() > 0)
	{
		m_aLines.GetLast().m_aPieces.SetSize(0);
	}

	m_cs.Leave();
}

