///////////////////////////////////////////////////////////////////////////////
// This file is part of the chuwin32 library
// Copyright (C) 2005/2008 ChuTeam
// For conditions of distribution and use, see copyright notice in chuwin32.h
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "gui.h"

using namespace chuwin32;

namespace chuwin32 {

void SetRect(RECT& rect, int x1, int y1, int x2, int y2)
{
	rect.left = x1;
	rect.top = y1;
	rect.right = x2;
	rect.bottom = y2;
}
bool IsRectNull(const RECT& rect)
{
	if( rect.left == 0 && rect.top == 0 && rect.right == 0 && rect.bottom == 0)
		return true;

	return false;
}

bool IsRectEmpty(const RECT& rect)
{
	int nWidth = rect.right - rect.left;
	int nHeight = rect.bottom - rect.top;
	if( nWidth <= 0 || nHeight <= 0)
		return true;

	return false;
}

int RectWidth(const RECT& rect)
{
	return rect.right - rect.left;
}

int RectHeight(const RECT& rect)
{
	return rect.bottom - rect.top;
}

bool IsPointInRect(const RECT& rect, POINT pt)
{
	if( rect.left <= pt.x && pt.x < rect.right)
	{
		if( rect.top <= pt.y && pt.y < rect.bottom)
		{
			return true;
		}
	}
	return false;
}

POINT operator - (POINT pt1, POINT pt2)
{
	POINT pt;
	pt.x = pt1.x - pt2.x;
	pt.y = pt1.y - pt2.y;
	return pt;
}

} // namespace chuwin32

////////////////////////////////////////////////////////////////////////////////////////////
void Window::CenterWindow(bool bAvoidHide)
{
	RECT rectParent;

	HDC hDC = ::GetDC(NULL);
	int nScreenWidth = ::GetDeviceCaps(hDC, HORZRES);
	int nScreenHeight = ::GetDeviceCaps(hDC, VERTRES);
	::ReleaseDC(NULL, hDC);

	HWND hParent = GetParent();
	if( hParent == NULL)
	{
		rectParent.top = 0;
		rectParent.left = 0;
		
		rectParent.right = nScreenWidth;
		rectParent.bottom = nScreenHeight;
	}
	else
	{
		::GetWindowRect(hParent, &rectParent);
	}
	RECT rect = GetWindowRect();
	
	const int nParentWidth = rectParent.right - rectParent.left;
	const int nParentHeight = rectParent.bottom - rectParent.top;

	int nWidth = rect.right - rect.left;
	int nHeight = rect.bottom - rect.top;
	
	int nX = rectParent.left + (nParentWidth - nWidth) / 2;
	int nY = rectParent.top + (nParentHeight - nHeight) / 2;
	
	if( bAvoidHide)
	{
		int nDiffXLeft = nX;
		int nDiffXRight = (nX + nWidth) - nScreenWidth;

		if( nDiffXLeft > 0 && nDiffXRight > 0)
		{
			nX -= nDiffXRight;
		}
		else if( nDiffXLeft < 0)
		{
			nX = 0;
		}

		int nDiffYBottom = (nY + nHeight) - nScreenHeight;
		if( nDiffYBottom > 0)
		{
			nY -= nDiffYBottom;
		}

	}
	MoveWindow(m_hWnd, nX, nY, nWidth, nHeight, TRUE);
}

bool Window::CreateEx(uint32 nExStyle, const wchar* pszClassName, const wchar* pszTitle, uint32 nStyle,
			  int x, int y, int w, int h, HWND hParent, HMENU hMenu)
{
	HINSTANCE hInstance = GetModuleHandle(NULL);

	m_hWnd = CreateWindowExW(
		nExStyle, // extended window style
		pszClassName,   // registered class name
		pszTitle,     // window name
		nStyle,      // window style
		x,    // horizontal position of window
		y,     // vertical position of window
		w, // window width
		h, // window height
		hParent,          // handle to parent or owner window
		hMenu,          // menu handle or child identifier
		hInstance,   // handle to application instance (Windows NT/2000/XP: This value is ignored)
		this           // window-creation data
		);

	return m_hWnd != 0;
}

bool Window::CreateEx(uint32 nExStyle, const wchar* pszClassName, const wchar* pszTitle, uint32 nStyle,
		  int x, int y, int w, int h, HWND hParent, int nId)
{
	return CreateEx(nExStyle, pszClassName, pszTitle, nStyle, x, y, w, h, hParent, HMENU( LongToHandle(nId)));
}

bool Window::CreateEx(uint32 nExStyle, const wchar* pszClassName, const wchar* pszTitle, uint32 nStyle,
		  const RECT& rect, HWND hParent, HMENU hMenu)
{
	return CreateEx(nExStyle, pszClassName, pszTitle, nStyle, 
		rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, hParent, hMenu);
}

bool Window::CreateEx(uint32 nExStyle, const wchar* pszClassName, const wchar* pszTitle, uint32 nStyle,
		  const RECT& rect, HWND hParent, int nId)
{
	return CreateEx(nExStyle, pszClassName, pszTitle, nStyle, rect, hParent, HMENU( LongToHandle(nId)));
}

//////////////////////
LRESULT CALLBACK Window::WndProcStatic(HWND hWnd, UINT nMsg, WPARAM wParam, LPARAM lParam)
{
	Window* pWin = null;

	if( nMsg == WM_NCCREATE)
	{
		LPCREATESTRUCT pCreateStruct = (LPCREATESTRUCT) lParam;
		pWin = (Window*) pCreateStruct->lpCreateParams;

		// The pointer must given with the CreateWindow
		ASSERT(pWin != NULL);

		::SetWindowLongPtr(hWnd, GWLP_USERDATA, PtrToUlong(pWin));
		pWin->m_hWnd = hWnd;
	}
	else
	{
		LONG nLong = ::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		pWin = (Window*) LongToPtr(nLong);
	}
	
	if( pWin == null)
	{
		// Can occur in very rare cases
		return ::DefWindowProc(hWnd, nMsg, wParam, lParam);
	}

	return pWin->WndProc(nMsg, wParam, lParam);
}

LRESULT Window::WndProc(UINT nMsg, WPARAM wParam, LPARAM lParam)
{
	return DefWindowProc(m_hWnd, nMsg, wParam, lParam);
}
